 /*tree36a.c
    maeyeda's 2-tree method to evaluate topological formulas using
    (minty's algorithm for spanning tree enumeration
    in symbolic network analysis, lin) */
#define gcc 0
#if gcc
#include <stdio.h>
#else
#include "..\include\stdio.h"
#endif


#define actmax 20		/* max active 2-ports */
#define pmax 60			/*max passive (one-port) edges */
#define emax (pmax+actmax)	/* max edges */
#define nmax 30			/* max nodes */
#define stk1 nmax		/* minty alg. stack */
#define stk2 nmax		/* traverse stack */

/* numerator and denominator indicators */
#define V     0
#define wijkl 1
#define wijij 2
#define wklkl 3
#define wklij 4
#define uijkl 5

/* test parameter masks */
#define INPUT_DUMP 1
#define RAW_TREES 2
#define BUFF_TREES 4
#define TREE_PERM 8
#define MIXED_UNITS 16
#define ADMITT_UNITS 32
#define PRINT_STATS 64
#define DEBUG_128 128
#define DEBUG_256 256
#define DEBUG_512 512
#define DEBUG_1024 1024
int denfn, numfn, inptno, evfn, numcntxt;

int tbufmax;
char *treebuf;

#define TREEBUFELEMENT char
#define TREEBUFROW (n + 1)
int tbufoverage;
int nbuf, nbuf1;
int branchtag[emax], impedancelist[emax], impedancecount;
int powermap[emax], fmap[emax];
int dtag;
int ni, nj, nk, nl, ni1, nj1, nk1, nl1, opampcnt, opamp, n, int_node, nodelist[nmax];
int p0[pmax], p1[pmax], p2[pmax];
int test;

int e, p, r, a, d0[emax], di1[emax], di2[emax], dv1[emax], dv2[emax];
int a0[actmax], a1[actmax], a2[actmax], a3[actmax], a4[actmax];
volatile int branches;
char branch[emax][8];



#define maxsym emax
#define maxterm emax
#define maxdegree nmax
int (*numtermlist)[maxsym + 2];
int (*denomtermlist)[maxsym + 2];
int (*numpoly)[maxdegree], (*denompoly)[maxdegree];
int term[maxsym], cancelfactor[maxsym], cancelcoeff, cancelsign, cancelexp;
int branchmap[maxsym];
int numtoterms;
int denomtoterms;
int termlistoverage;
int numdegterms[maxdegree];
int denomdegterms[maxdegree];
int nummaxterms, nummaxpoly, denommaxterms, denommaxpoly;
int iit, i_nt, ivstk[stk1][nmax], iestk[stk1][emax], istate, itree[nmax];
int vit, vnt, vvstk[stk1][nmax], vestk[stk1][emax], vstate, vtree[nmax];
int mn, me, mr, mq;

#define NUM_VARS &numtermlist,&numpoly,numdegterms,&numtoterms,&nummaxterms,&nummaxpoly
#define DENOM_VARS &denomtermlist,&denompoly,denomdegterms,&denomtoterms,&denommaxterms,&denommaxpoly
int nodemap (int node);
int internal_node (void);
void release (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly);
void evaluate_fn (int e, int a, int r, int sign, int numerator, int denominator);
#define ev_fn(s,n,d) evaluate_fn(e,a,r, s,n,d)
void sii_eval(int niin, int njin, int nkin, int nlin);
void sij_eval(int niin, int njin, int nkin, int nlin);
void evaluate_s_parameters(int e, int a, int r);
void print_treebuf (int nn);
void treecheck (int r, int *d1, int *d2, int *tree, int *perm);
int product (int minty_nodes, int edges, int actual_nodes, int form, int br);
void iminty (void);
void vminty (void);
void combineinit (void);
void init (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly);
int resize(void ** ptr, int * count, int module);
int getexpr (int sign, int deg, int factors, int *factorlist, int edges, int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly);
void printexp (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly);
void makecancellationlist (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly);
int getsym (int i, int edges);
int correctunits (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly);
int fixup (void);
void eval_branchtag (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly);

int 
main ()
{
    int i, j, k, scan, node1, node2, node3, node4, int1, int2, ch0, ch1,
      f;
    char string[132], ch;

    printf ("program file name tree36a.c\n\r");
    inptno = 0;
    for (;;) {

	printf ("\n\r\n\rinput data:\n\r\n\r");

	inptno++;

	test = 0;
	p = 1;			/* first edge is always dummy */
	branches = 1;
	n = 0;
	int_node = 999;
	a = 0;
	f = 0;


	for (i = 0; i < nmax; i++)
	    nodelist[i] = 0;

	for (scan = 1; scan == 1;) {
	    if (scanf ("%s", &branch[branches + 1][0]) == EOF)
		return (0);
	    printf ("%s", &branch[branches + 1][0]);
	    ch0 = branch[branches + 1][0];
	    ch1 = branch[branches + 1][1];
	    if (ch0 == '{') {
		scanf ("%c", &ch);
		while (ch != '}') {
		    printf ("%c", ch);
		    scanf ("%c", &ch);
		};
		printf ("}\n\r");
	    } else if ((((ch0 == 'g') || (ch0 == 'G') || (ch0 == 'r') || (ch0 == 'R')) && ((ch1 == 'm') || (ch1 == 'M'))) ||
		       (ch0 == 'o') || (ch0 == 'O') || (ch0 == 'u') || (ch0 == 'U') || (ch0 == 'b') || (ch0 == 'B')) {
		scanf (" %u %u %u %u", &node1, &node2, &node3, &node4);
		printf (" %u %u %u %u\n\r", node1, node2, node3, node4);
		switch (ch0) {
		case 'g':
		case 'G':
		case 'o':
		case 'O':
		    branches++;
		    a++;
		    a0[a] = branches;
		    a1[a] = nodemap (node1);
		    a2[a] = nodemap (node2);
		    a3[a] = nodemap (node3);
		    a4[a] = nodemap (node4);
		    break;
		case 'r':
		case 'R':

		    branches++;
		    p++;
		    p0[p] = branches;
		    p1[p] = nodemap (node3);
		    p2[p] = internal_node ();

		    branches++;
		    a++;
		    a0[a] = branches;
		    a1[a] = internal_node ();
		    a2[a] = nodemap (node2);
		    a3[a] = nodemap (node4);
		    a4[a] = p2[p];
		    branch[branches][0] = 'a';
		    branch[branches][1] = 0;

		    branches++;
		    a++;
		    a0[a] = branches;
		    a1[a] = nodemap (node1);
		    a2[a] = nodemap (node2);
		    a3[a] = nodemap (node2);
		    a4[a] = a1[a - 1];
		    branch[branches][0] = 'o';
		    branch[branches][1] = 0;


		    branches++;
		    a++;
		    a0[a] = branches;
		    a1[a] = p2[p];
		    a2[a] = nodemap (node4);
		    a3[a] = nodemap (node4);
		    a4[a] = nodemap (node3);
		    branch[branches][0] = 'o';
		    branch[branches][1] = 0;

		    branches++;
		    p++;
		    p0[p] = branches;
		    p1[p] = nodemap (node1);
		    p2[p] = a1[a - 2];
		    branch[branches][0] = 'a';
		    branch[branches][1] = 0;

		    break;

		case 'u':
		case 'U':
		    branches++;
		    a++;
		    a0[a] = branches;
		    a1[a] = internal_node ();
		    a2[a] = nodemap (node2);
		    a3[a] = nodemap (node3);
		    a4[a] = nodemap (node4);

		    branches++;
		    a++;
		    a0[a] = branches;
		    a1[a] = nodemap (node1);
		    a2[a] = nodemap (node2);
		    a3[a] = nodemap (node2);
		    a4[a] = a1[a - 1];
		    branch[branches][0] = 'o';
		    branch[branches][1] = 0;

		    branches++;
		    p++;
		    p0[p] = branches;
		    p1[p] = a1[a - 1];
		    p2[p] = nodemap (node1);
		    branch[branches][0] = 'a';
		    branch[branches][1] = 0;

		    break;

		case 'b':
		case 'B':
		    branches++;
		    a++;
		    a0[a] = branches;
		    a1[a] = nodemap (node2);
		    a2[a] = nodemap (node1);
		    a3[a] = internal_node ();
		    a4[a] = nodemap (node4);

		    branches++;
		    a++;
		    a0[a] = branches;
		    a1[a] = a3[a - 1];
		    a2[a] = nodemap (node4);
		    a3[a] = nodemap (node4);
		    a4[a] = nodemap (node3);
		    branch[branches][0] = 'o';
		    branch[branches][1] = 0;

		    branches++;
		    p++;
		    p0[p] = branches;
		    p1[p] = nodemap (node3);
		    p2[p] = a3[a - 1];
		    branch[branches][0] = 'a';
		    branch[branches][1] = 0;

		    break;

		}
	    } else if ((ch0 == 'm') || (ch0 == 'M')) {
		scanf (" %u %u %u %u", &node1, &node2, &node3, &node4);
		printf (" %u %u %u %u\n\r", node1, node2, node3, node4);

		int1 = internal_node ();
		int2 = internal_node ();

		branches++;
		p++;
		p0[p] = branches;
		p1[p] = nodemap (node3);
		p2[p] = internal_node ();
		k = branches;

		branches++;
		a++;
		a0[a] = branches;
		a1[a] = internal_node ();
		a2[a] = int2;
		a3[a] = int1;
		a4[a] = p2[p];
		branch[branches][0] = 'a';
		branch[branches][1] = 0;

		branches++;
		a++;
		a0[a] = branches;
		a1[a] = nodemap (node1);
		a2[a] = int2;
		a3[a] = int2;
		a4[a] = a1[a - 1];
		branch[branches][0] = 'o';
		branch[branches][1] = 0;


		branches++;
		a++;
		a0[a] = branches;
		a1[a] = p2[p];
		a2[a] = int1;
		a3[a] = int1;
		a4[a] = nodemap (node3);
		branch[branches][0] = 'o';
		branch[branches][1] = 0;

		branches++;
		p++;
		p0[p] = branches;
		p1[p] = nodemap (node1);
		p2[p] = a1[a - 2];
		branch[branches][0] = 'a';
		branch[branches][1] = 0;


/* second part */

		branches++;
		p++;
		for (i = 0; (branch[branches][i] = branch[k][i]) != 0; i++);
		p0[p] = branches;
		p1[p] = int2;
		p2[p] = internal_node ();

		branches++;
		a++;
		a0[a] = branches;
		a1[a] = internal_node ();
		a2[a] = nodemap (node4);
		a3[a] = nodemap (node2);
		a4[a] = p2[p];
		branch[branches][0] = 'a';
		branch[branches][1] = 0;

		branches++;
		a++;
		a0[a] = branches;
		a1[a] = int1;
		a2[a] = nodemap (node4);
		a3[a] = nodemap (node4);
		a4[a] = a1[a - 1];
		branch[branches][0] = 'o';
		branch[branches][1] = 0;


		branches++;
		a++;
		a0[a] = branches;
		a1[a] = p2[p];
		a2[a] = nodemap (node2);
		a3[a] = nodemap (node2);
		a4[a] = int2;
		branch[branches][0] = 'o';
		branch[branches][1] = 0;

		branches++;
		p++;
		p0[p] = branches;
		p1[p] = int1;
		p2[p] = a1[a - 2];
		branch[branches][0] = 'a';
		branch[branches][1] = 0;



	    } else if (((ch0 == 'g') || (ch0 == 'G')) && ((ch1 == 'y') || (ch1 == 'Y'))) {
		scanf (" %u %u %u %u", &node1, &node2, &node3, &node4);
		printf (" %u %u %u %u\n\r", node1, node2, node3, node4);

		branches++;
		a++;
		a0[a] = branches;
		a1[a] = nodemap (node1);
		a2[a] = nodemap (node2);
		a3[a] = nodemap (node3);
		a4[a] = nodemap (node4);

		branches++;
		for (i = 0; (branch[branches][i] = branch[branches - 1][i]) != 0; i++);

		a++;
		a0[a] = branches;
		a1[a] = nodemap (node4);
		a2[a] = nodemap (node3);
		a3[a] = nodemap (node1);
		a4[a] = nodemap (node2);



	    } else if ((ch0 == 'f') || (ch0 == 'F')) {
		if (f == 1) {
		    printf ("error-- more than one port indicator is used\n\r");
		    return (1);
		};
		f = 1;
		scanf (" %u %u %u %u", &ni1, &nj1, &nk1, &nl1);
		printf (" %u %u %u %u\n\r", ni1, nj1, nk1, nl1);
		ni = nodemap (ni1);
		nj = nodemap (nj1);
		nk = nodemap (nk1);
		nl = nodemap (nl1);
	    } else if ((ch0 == 'g') || (ch0 == 'G') ||
		       (ch0 == 'z') || (ch0 == 'Z') ||
		       (ch0 == 'y') || (ch0 == 'Y') ||
		       (ch0 == 'r') || (ch0 == 'R') ||
		       (ch0 == 'l') || (ch0 == 'L') ||
		       (ch0 == 'c') || (ch0 == 'C')) {
		scanf (" %u %u", &node1, &node2);
		printf (" %u %u\n\r", node1, node2);
		branches++;
		p++;
		p0[p] = branches;
		p1[p] = nodemap (node1);
		p2[p] = nodemap (node2);
	    } else if ((ch0 == 'e') || (ch0 == 'E')) {
/* dummy edge (handle) and extra node */
		p0[1] = 1;
		p1[1] = n + 1;
		p2[1] = 1;
		branch[1][0] = 'x';
		branch[1][1] = 0;
		n++;
		scan = 0;
	    } else if ((ch0 == 'h') || (ch0 == 'H')) {
		printf ("\n\rhelp:\n\r");
		printf ("comments (on their own lines) are bracked:  {comments... }\n\r");
                printf ("program written by Jeff Furman 5-4-2001 vers 36a\n\r");
		printf ("one-port (two-terminal) passive elements:\n\r");
		printf ("r,l,c,g,y,z (lower case or upper case) prefix, null up to 8 character suffix,\n\r");
		printf ("whitespace, first node#(0-999),whitespace,second node#(0-999) examples--\n\r");
		printf ("r 0 4\n\r");
		printf ("L23a 7 41\n\r");
		printf ("g2 3 8\n\r");
		printf ("(gm, gy, and rm are reserved for two-port elements)\n\r");
		printf ("two-port (four-terminal) active elements:\n\r");
		printf ("gm,gy,o,u,b,rm,m (lower case or upper case) prefix, null up to 8 character suffix,\n\r");
		printf ("whitespace, +output node#, whitespace, -output node#, whitespace, +input node#, whitespace, -input node#,\n\r");
		printf ("positive current enters +nodes.\n\r");
		printf ("gm denotes voltage controlled current source,\n\r");
		printf ("o denotes ideal op amp,\n\r");
		printf ("u (mu) denotes voltage controlled voltage source,\n\r");
		printf ("b (beta) denotes current controlled current source,\n\r");
		printf ("rm denotes current controlled voltage source,\n\r");
		printf ("gy denotes a gyrator, and\n\r");
		printf ("m denotes a mutual inductance\n\r");
		printf ("examples--\n\r");
		printf ("gm2 1 2 3 4\n\r");
		printf ("U3 3 0 7 5\n\r");
		printf ("o 6 3 2 9\n\r");
		printf ("O 3 1 3 4\n\r");
		printf ("f (lower case or upper case),\n\r");
		printf ("whitespace, +port1 node#, whitespace, -port1 node#, whitespace, +port2 node#, whitespace, -port2 node#\n\r");
		printf ("f 1 0 2 0\n\r");
		printf ("e or end  terminates input\n\r");
		printf ("t # before end line shows intermediate results: 1-input data dump,2-raw tree listing,4-buffered tree listing,\n\r8-matched trees and permutation listing,16-mixed unit trees,32-admittance trees,64-resource usage\n\r");
		printf ("resource burden of input element types:\n\r");
		printf ("elem prefix, branch name list, node list, passive edges, active edges\n\r");
		printf ("r,l,c,g,y,z        1           no extra         1             0\n\r");
		printf (" GM, O             1           no extra         0             1\n\r");
		printf (" GY                2           no extra         0             2\n\r");
		printf ("  U, B             3            1 extra         1             2\n\r");
		printf (" RM                5            2 extra         2             3\n\r");
		printf ("  M               10            6 extra         4             6\n\r");
		printf (" the extra nodes are invisible externally\n\r");
                printf("\n\r");
                printf("two-port parameter notation and equivalences:\n\r");
                printf("for ports expressed as f <i> <j> <k> <l>\n\r");
                printf("where <i> <j> <k> <l> are numeric node numbers labeling the port terminals,\n\r");
                printf("then port 1 is denoted by (i,j) and port 2 is denoted by (k,l)\n\r");
                printf("examples: y(i,j;k,l) == y12\n\r");
                printf("g(k,l;i,j) == g21 etc.\n\r");
                printf("y denotes short circuit admittance parameters,\n\r");
                printf("z denotes open  circuit impedance  parameters,\n\r");
                printf("h denotes hybrid parameters,\n\r");
                printf("g denotes inverse hybrid parameters,\n\r");
                printf("A,B,C,D denotes forward chain or transmission parameters,\n\r");
                printf("a,b,c,d denotes reverse chain parameters,\n\r");
                printf("s denotes scattering parameters,\n\r");
                printf("F denotes scattering forward chain parameters,\n\r");
                printf("these parameters are directly computed:\n\r");
                printf("g21, h12, h21, g12, y11, y21, y12, y22, z11, z21, z12, z22,\n\r");
                printf("s11, s21, s12, s22, F11, F21, F12, F22.\n\r");
                printf("from these, the remaining g, h, ABCD, and abcd parameters are evaluated by\n\r");
                printf("using reciprocals (exchanging numerator and denominator) and possibly negating\n\r");
                printf("either numerator or denominator:\n\r");
                printf("g11 == 1/z11; g22 == 1/y22; h11 == 1/y11; h22 == 1/z22;\n\r");
                printf("A == 1/g21; B == -1/y21; C == 1/z21; D == -1/h21;\n\r");
                printf("a == 1/h12; b == -1/y12; c == 1/z12; d == -1/g12.\n\r");
                printf("\n\r");
                printf("references:\n\r");
                printf("Huelsman, L. P.; \"Circuits, Matrices, and Linear Vector Spaces\"; pp. 57-63\n\r");
                printf("Weinberg, L.; Chapter 4 in Shea, R. F., ed.; \"Amplifier Handbook\" pp. 4-27 - 4-31.\n\r");
                printf("Lin, Pen-Min; \"Symbolic Network Analysis\"; pp. 115-117, pp. 213-219\n\r");
                printf("Mayeda, W.; \"Graph Theory\"; ch. 7 and 8; pp. 252-341\n\r");
                printf("\n\r");
		printf (" currently configured resource limits:\n\r");
		printf ("branch name list %d\n\r", emax);
		printf ("node list %d\n\r", nmax);
		printf ("passive edges %d\n\r", pmax);
		printf ("active edges %d\n\r", actmax);
		printf ("{sample sallen-key 3rd order lowpass:}\n\r");
		printf ("{optional intermediate results}\n\r");
		printf ("t 64\n\r");
		printf ("r1 1 2\n\r");
		printf ("r2 2 3\n\r");
		printf ("r3 3 4\n\r");
		printf ("c 2 5\n\r");
		printf ("c 3 6\n\r");
		printf ("c 4 5\n\r");
		printf ("{vcvs here}\n\r");
		printf ("u 6 5 4 5\n\r");
		printf ("f 6 5 1 5\n\r");
		printf ("end\n\r");
		return (0);
	    } else if ((ch0 == 't') || (ch0 == 'T')) {
		scanf (" %u", &test);
		printf (" %u\n\r", test);
	    } else {
		printf ("unrecognized element prefix-- '%s': must be r,l,c,g,y,z,gm,gy,o,u,b,rm,m,f,e\n\r", &branch[branches + 1][0]);
		printf ("h as input prints help\n\r");
		return (1);
	    };



	    if (p >= pmax) {
		printf ("number of one-port elements(= %d) exceeds limit of %d\n\r or improper input applied\n\r", p, pmax - 1);
		return (1);
	    };

	    if (a >= actmax) {
		printf ("number of two-port elements(= %d) exceeds limit of %d\n\r or improper input applied\n\r", a, actmax - 1);
		return (1);
	    };


	} printf ("\n\r\n\r\n\r");

	if (f == 0) {
	    printf ("please provide a port node indication: f (lower case or upper case),\n\r");
	    printf ("whitespace, +port1 node#, whitespace, -port1 node#, whitespace, +port2 node#, whitespace, -port2 node#\n\
r");
	    printf ("example:\n\r");
	    printf ("f 1 0 2 4\n\r");

	    return (1);
	};
	if ((test & PRINT_STATS) != 0) {
	    printf (" currently configured resource limits:\n\r");
	    printf ("branch name list limit:%d, used:%d\n\r", emax, branches);
	    printf ("node list limit:%d, used:%d\n\r", nmax, n);
	    printf ("passive edges limit:%d, used:%d\n\r", pmax, p);
	    printf ("active edges limit:%d, used:%d\n\r", actmax, a);
	};

	e = p;
	r = n - 1;


	if ((test & INPUT_DUMP) != 0) {
	    printf ("Internal representation of data including generated elements and reassigned node#s:\n");
	    printf ("edge,name,node#'s...\n\r");
	    for (i = 1, j = 1; i <= p; i++, j++) {
		printf ("%3d %s %d %d\n", j, &branch[p0[i]][0], p1[i], p2[i]);
	    };
	    for (i = 1; i <= a; i++, j++) {
		printf ("%3d %s %d %d %d %d\n", j, &branch[a0[i]][0], a1[i], a2[i], a3[i], a4[i]);
	    };
	    printf ("%3d f %d %d %d %d\n\n", j, ni, nj, nk, nl);
	    printf ("internal node#  input node#:\n\r");
	    for (i = 1; i <= n; i++)
		printf ("         %5d    %d\n\r", i, nodelist[i]);
	    printf ("\n\r\n\redge namelist: index, value\n\r");
	    for (i = 1; i <= branches; i++)
		printf ("%3d, %s\n\r", i, &branch[i][0]);
	    printf ("\n\r\n\r");
	};

/* numerator and denominator indicators
   #define V     0
   #define wijkl 1
   #define wijij 2
   #define wklkl 3
   #define wklij 4
   #define uijkl 5
 */

/*
   transfer ratios:
   Vkl/Vij = wijkl/wijij
   Ikl/Iij = -wijkl/wklkl
   open circuit admittance functions:
   Iij/Vij = V/wijij
   Ikl/Vij = V/wklij
   Iij/Vkl = V/wijkl
   Ikl/Vkl = V/wklkl
   short circuit admittance functions:
   Iij/Vij = wklkl/uijkl
   Iij/Vkl = -wklij/uijkl
   Ikl/Vij = -wijkl/uijkl
   Ikl/Vkl = wijij/uijkl
 */


	evfn = 0;

        printf (" Function:\n\n g( %d, %d; %d, %d) = v( %d, %d)/v( %d, %d) for i( %d, %d)=0\n", nk1, nl1, ni1, nj1, nk1, nl1, ni1, nj1, nk1, nl1);
	ev_fn (1, wijkl, wijij);

        printf (" Function:\n\n h( %d, %d; %d, %d) = v( %d, %d)/v( %d, %d) for i( %d, %d)=0\n", ni1, nj1, nk1, nl1, ni1, nj1, nk1, nl1, ni1, nj1);
	ev_fn (1, wklij, wklkl);

        printf (" Function:\n\n h( %d, %d; %d, %d) = i( %d, %d)/i( %d, %d) for v( %d, %d)=0\n", nk1, nl1, ni1, nj1, nk1, nl1, ni1, nj1, nk1, nl1);
	ev_fn (-1, wijkl, wklkl);

        printf (" Function:\n\n g( %d, %d; %d, %d) = i( %d, %d)/i( %d, %d) for v( %d, %d)=0\n", ni1, nj1, nk1, nl1, ni1, nj1, nk1, nl1, ni1, nj1);
	ev_fn (-1, wklij, wijij);

        printf (" Function:\n\n y( %d, %d; %d, %d) = i( %d, %d)/v( %d, %d) for V( %d, %d)=0\n"
		,ni1, nj1, ni1, nj1, ni1, nj1, ni1, nj1, nk1, nl1);
	ev_fn (1, wklkl, uijkl);

        printf (" Function:\n\n y( %d, %d; %d, %d) = i( %d, %d)/v( %d, %d) for V( %d, %d)=0\n"
		,nk1, nl1, ni1, nj1, nk1, nl1, ni1, nj1, nk1, nl1);
	ev_fn (-1, wijkl, uijkl);

        printf (" Function:\n\n y( %d, %d; %d, %d) = i( %d, %d)/v( %d, %d) for V( %d, %d)=0\n"
		,ni1, nj1, nk1, nl1, ni1, nj1, nk1, nl1, ni1, nj1);
	ev_fn (-1, wklij, uijkl);

        printf (" Function:\n\n y( %d, %d; %d, %d) = i( %d, %d)/v( %d, %d) for V( %d, %d)=0\n"
		,nk1, nl1, nk1, nl1, nk1, nl1, nk1, nl1, ni1, nj1);
	ev_fn (1, wijij, uijkl);

        printf (" Function:\n\n z( %d, %d; %d, %d) = v( %d, %d)/i( %d, %d) for I( %d, %d)=0\n"
		,ni1, nj1, ni1, nj1, ni1, nj1, ni1, nj1, nk1, nl1);	ev_fn (1, wijij, V);

        printf (" Function:\n\n z( %d, %d; %d, %d) = v( %d, %d)/i( %d, %d) for I( %d, %d)=0\n"
		,nk1, nl1, ni1, nj1, nk1, nl1, ni1, nj1, nk1, nl1);
	ev_fn (1, wijkl, V);

        printf (" Function:\n\n z( %d, %d; %d, %d) = v( %d, %d)/i( %d, %d) for I( %d, %d)=0\n"
		,ni1, nj1, nk1, nl1, ni1, nj1, nk1, nl1, ni1, nj1);
	ev_fn (1, wklij, V);

        printf (" Function:\n\n z( %d, %d; %d, %d) = v( %d, %d)/i( %d, %d) for I( %d, %d)=0\n"
		,nk1, nl1, nk1, nl1, nk1, nl1, nk1, nl1, ni1, nj1);
	ev_fn (1, wklkl, V);

        evaluate_s_parameters( e, a, r);


    };
}
 int 
nodemap (int node)
{
    int i;

    if (node > 999) {
	printf ("node label %d must be less than 1000\n\r", node);
	return (1);
    };

    for (i = 1; i <= n; i++) {
	if (nodelist[i] == node)
	    return (i);
    };
    if (n >= nmax) {
	printf ("number of nodes(= %d) exceeds limit of %d\n\r or improper input applied\n\r", n, nmax - 1);
	return (1);
    };
    n++;
    nodelist[n] = node;
    return (n);
}
 int 
internal_node (void)
{
    if (n >= nmax) {
	printf ("number of nodes(= %d) exceeds limit of %d\n\r or improper input applied\n\r", n, nmax - 1);
	return (1);
    };

    n++;
    int_node++;
    nodelist[n] = int_node;
    return (n);
}
/* -1<=>nk,-2<=>nl,-3<=>nj,-4<=>ni */
#define nodesubs(nnn) ((nnn == -1)?(nksave):((nnn == -2)?(nlsave):((nnn == -3)?(njsave):((nnn == -4)?(nisave):(n - 1 + nnn)))))

void evaluate_s_parameters(int ee, int aa, int rr)
{
int esave, asave, rsave, nsave, psave, brsave;
int nisave, njsave, nksave, nlsave;
int i, j, k;

int opamp_list[][4] = {
                        {2, -3, 1, 2},
                        {3, -3, -4, 4},
                        {6, -2, 5, 6},
                        {7, -2, -1, 8},
                        {0, 0, 0, 0},
                      };

/* vcis unused in embedding circuit */
int vcis_list[][4] = {
                        {0, 0, 0, 0},
                      };

int r_list[][2] = {
                    {1, 3},
                    {4, 3},
                    {4, -3},
                    {5, 7},
                    {7, 8},
                    {8, -2},
                    {0, 0},
                      };
int ref_list[][2] = {
                    {2, -4},
                    {6, -1},
                    {0, 0},
                   };


esave = ee;
asave = aa;
rsave = rr;
nsave = n;
psave = p;
brsave = branches;

nisave = ni;
njsave = nj;
nksave = nk;
nlsave = nl;

ni = nodesubs(1);
nk = nodesubs(5);

for(i=0;opamp_list[i][0] != 0;i++){


		branches++;
		a++;
                a0[a] = branches;
                a1[a] = nodesubs(opamp_list[i][0]);
                a2[a] = nodesubs(opamp_list[i][1]);
                a3[a] = nodesubs(opamp_list[i][2]);
                a4[a] = nodesubs(opamp_list[i][3]);
		branch[branches][0] = 'o';
		branch[branches][1] = 0;


};


for(i=0;vcis_list[i][0] != 0;i++){


		branches++;
		a++;
                a0[a] = branches;
                a1[a] = nodesubs(vcis_list[i][0]);
                a2[a] = nodesubs(vcis_list[i][1]);
                a3[a] = nodesubs(vcis_list[i][2]);
                a4[a] = nodesubs(vcis_list[i][3]);
                branch[branches][0] = 'a';
		branch[branches][1] = 0;


};

for(i=0;r_list[i][0] != 0;i++){

		branches++;
		p++;
		p0[p] = branches;
                p1[p] = nodesubs(r_list[i][0]);
                p2[p] = nodesubs(r_list[i][1]);
                branch[branches][0] = 'a';
		branch[branches][1] = 0;


};

for(i=0;ref_list[i][0] != 0;i++){

		branches++;
		p++;
		p0[p] = branches;
                p1[p] = nodesubs(ref_list[i][0]);
                p2[p] = nodesubs(ref_list[i][1]);
                branch[branches][0] = 'Y';
                branch[branches][1] = '0';
                branch[branches][2] = 0;


};

n = n + 7;

/* dummy edge (handle) and extra node */
		p0[1] = 1;
		p1[1] = n + 1;
		p2[1] = 1;
		branch[1][0] = 'x';
		branch[1][1] = 0;
		n++;

	e = p;
	r = n - 1;

	if ((test & INPUT_DUMP) != 0) {
	    printf ("Internal representation of data including generated elements and reassigned node#s:\n");
            printf ("this includes embedding circuitry for evaluating scattering parameters.\n\r");
	    printf ("edge,name,node#'s...\n\r");
	    for (i = 1, j = 1; i <= p; i++, j++) {
		printf ("%3d %s %d %d\n", j, &branch[p0[i]][0], p1[i], p2[i]);
	    };
	    for (i = 1; i <= a; i++, j++) {
		printf ("%3d %s %d %d %d %d\n", j, &branch[a0[i]][0], a1[i], a2[i], a3[i], a4[i]);
	    };
	    printf ("%3d f %d %d %d %d\n\n", j, ni, nj, nk, nl);
/*
	    printf ("internal node#  input node#:\n\r");
	    for (i = 1; i <= n; i++)
		printf ("         %5d    %d\n\r", i, nodelist[i]);
*/
	    printf ("\n\r\n\redge namelist: index, value\n\r");
	    for (i = 1; i <= branches; i++)
		printf ("%3d, %s\n\r", i, &branch[i][0]);
	    printf ("\n\r\n\r");
	};



        printf (" Function:\n\n s( %d, %d; %d, %d) for Y0 termination\n", ni1, nj1, ni1, nj1);

        ev_fn (-1, wklkl, uijkl);

        printf (" Function:\n\n s( %d, %d; %d, %d) for Y0 termination\n", ni1, nj1, nk1, nl1);

        ev_fn (1, wklij, uijkl);

        printf (" Function:\n\n s( %d, %d; %d, %d) for Y0 termination\n", nk1, nl1, ni1, nj1);

        ev_fn (1, wijkl, uijkl);

        printf (" Function:\n\n s( %d, %d; %d, %d) for Y0 termination\n", nk1, nl1, nk1, nl1);

        ev_fn (-1, wijij, uijkl);



        printf (" Function:\n\n F( %d, %d; %d, %d) for Y0 termination\n", ni1, nj1, ni1, nj1);

        ev_fn (-1, V, wijkl);

        printf (" Function:\n\n F( %d, %d; %d, %d) for Y0 termination\n", ni1, nj1, nk1, nl1);

        ev_fn (-1, wklkl, wijkl);

        printf (" Function:\n\n F( %d, %d; %d, %d) for Y0 termination\n", nk1, nl1, ni1, nj1);

        ev_fn (1, wijij, wijkl);

        printf (" Function:\n\n F( %d, %d; %d, %d) for Y0 termination\n", nk1, nl1, nk1, nl1);

        ev_fn (1, uijkl, wijkl);

e = esave;
a = asave;
r = rsave;
n = nsave;
p = psave;
branches = brsave;

ni = nisave;
nj = njsave;
nk = nksave;
nl = nlsave;

/* restore dummy edge (handle) and extra node */
		p0[1] = 1;
                p1[1] = n;
		p2[1] = 1;
		branch[1][0] = 'x';
		branch[1][1] = 0;

}

 void 
evaluate_fn (int e, int a, int r, int sign, int numerator, int denominator)
{
    int i, j, rc, passive_edges, minty_nodes, actual_nodes, active_edges,
      numerator_form, denominator_form;

    evfn++;

    denfn = denominator;
    numfn = numerator;


    opampcnt = 0;
    minty_nodes = r + 1;
    actual_nodes = r;
    passive_edges = e;
    dtag = 0;

    for (i = 0; i <= (e + a); i++)
	branchtag[i] = 0;


    for (i = 0; i < maxsym; i++)
	cancelfactor[i] = 0x7fffffff;

    cancelcoeff = 0;
    cancelsign = -1;
    cancelexp = 0x7fffffff;

    for (i = 1; i <= e; i++) {
	d0[i] = p0[i];
	dv1[i] = p1[i];
	dv2[i] = p2[i];
    };

    for (i = 1; i <= a; i++) {
	d0[e + i] = a0[i];
	dv1[e + i] = a1[i];
	dv2[e + i] = a2[i];
    };

    combineinit ();


    switch (numerator) {
    case V:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("V:\n\r");
	active_edges = a;
	numerator_form = 1;
	for (i = 1; i <= a; i++) {
	    j = dv1[e + i];
	    dv1[e + i] = dv2[e + i];
	    dv2[e + i] = j;
	};
	break;

    case wijkl:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("wijkl:\n\r");
	active_edges = a + 1;
	numerator_form = 0;
	dv1[e + a + 1] = ni;
	dv2[e + a + 1] = nj;
	branch[branches + 1][0] = 'f';
	branch[branches + 1][1] = 0;
	d0[e + a + 1] = branches + 1;
	fmap[e + a + 1] = 3;
	break;

    case wijij:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("wijij:\n\r");
	active_edges = a + 1;
	numerator_form = 0;
	dv1[e + a + 1] = ni;
	dv2[e + a + 1] = nj;
	d0[e + a + 1] = branches + 1;
	branch[branches + 1][0] = 'f';
	branch[branches + 1][1] = 0;
	fmap[e + a + 1] = 3;
	break;

    case wklkl:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("wklkl:\n\r");
	active_edges = a + 1;
	numerator_form = 0;
	dv1[e + a + 1] = nk;
	dv2[e + a + 1] = nl;
	d0[e + a + 1] = branches + 1;
	branch[branches + 1][0] = 'f';
	branch[branches + 1][1] = 0;
	fmap[e + a + 1] = 3;
	break;

    case wklij:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("wklij:\n\r");
	active_edges = a + 1;
	numerator_form = 0;
	dv1[e + a + 1] = nk;
	dv2[e + a + 1] = nl;
	branch[branches + 1][0] = 'f';
	branch[branches + 1][1] = 0;
	d0[e + a + 1] = branches + 1;
	fmap[e + a + 1] = 3;
	break;

    case uijkl:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("uijkl:\n\r");
	active_edges = a + 2;
	numerator_form = 2;
	dv1[e + a + 1] = ni;
	dv2[e + a + 1] = nj;
	d0[e + a + 1] = branches + 1;
	branch[branches + 1][0] = 'f';
	branch[branches + 1][1] = '1';
	branch[branches + 1][2] = 0;
	fmap[e + a + 1] = 1;
	dv1[e + a + 2] = nk;
	dv2[e + a + 2] = nl;
	d0[e + a + 2] = branches + 2;
	branch[branches + 2][0] = 'f';
	branch[branches + 2][1] = '2';
	branch[branches + 2][2] = 0;
	fmap[e + a + 2] = 2;
	break;

    };

    for (i = 1; i <= (passive_edges + active_edges); i++) {
	di1[i] = dv1[i];
	di2[i] = dv2[i];
    };




    switch (numerator) {
    case V:
	break;

    case wijkl:
	di1[e + a + 1] = (sign == 1) ? nk : nl;
	di2[e + a + 1] = (sign == 1) ? nl : nk;
	break;

    case wijij:
	if (sign == -1) {
	    j = di1[e + a + 1];
	    di1[e + a + 1] = di2[e + a + 1];
	    di2[e + a + 1] = j;
	};
	break;

    case wklkl:
	if (sign == -1) {
	    j = di1[e + a + 1];
	    di1[e + a + 1] = di2[e + a + 1];
	    di2[e + a + 1] = j;
	};
	break;

    case wklij:
	di1[e + a + 1] = (sign == 1) ? ni : nj;
	di2[e + a + 1] = (sign == 1) ? nj : ni;
	break;

    case uijkl:
	break;

    };

    for (i = 1; i <= a; i++) {
	di1[e + i] = a3[i];
	di2[e + i] = a4[i];
    };




    if ((rc = product (minty_nodes, passive_edges + active_edges, actual_nodes,
		       numerator_form, e + a)) != 0) {
	printf ("insufficient memory to buffer numerator common trees; remainder of function evaluation skipped\n\r");
	return;
    };



    for (i = 1; i <= a; i++) {
	dv1[e + i] = a1[i];
	dv2[e + i] = a2[i];
    };


    switch (numerator) {
    case V:
	break;

    case wijkl:
	break;

    case wijij:
	break;

    case wklkl:
	break;

    case wklij:
	break;

    case uijkl:
	break;

    };





    dtag = 1;


    switch (denominator) {
    case V:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("V:\n\r");
	active_edges = a;
	denominator_form = 1;
	break;

    case wijkl:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("wijkl:\n\r");
	active_edges = a + 1;
	denominator_form = 0;
	dv1[e + a + 1] = ni;
	dv2[e + a + 1] = nj;
	d0[e + a + 1] = branches + 1;
	branch[branches + 1][0] = 'f';
	branch[branches + 1][1] = 0;
	fmap[e + a + 1] = 3;
	break;

    case wijij:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("wijij:\n\r");
	active_edges = a + 1;
	denominator_form = 0;
	dv1[e + a + 1] = ni;
	dv2[e + a + 1] = nj;
	d0[e + a + 1] = branches + 1;
	branch[branches + 1][0] = 'f';
	branch[branches + 1][1] = 0;
	fmap[e + a + 1] = 3;
	break;

    case wklkl:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("wklkl:\n\r");
	active_edges = a + 1;
	denominator_form = 0;
	dv1[e + a + 1] = nk;
	dv2[e + a + 1] = nl;
	d0[e + a + 1] = branches + 1;
	branch[branches + 1][0] = 'f';
	branch[branches + 1][1] = 0;
	fmap[e + a + 1] = 3;
	break;


    case wklij:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("wklij:\n\r");
	active_edges = a + 1;
	denominator_form = 0;
	dv1[e + a + 1] = nk;
	dv2[e + a + 1] = nl;
	d0[e + a + 1] = branches + 1;
	branch[branches + 1][0] = 'f';
	branch[branches + 1][1] = 0;
	fmap[e + a + 1] = 3;
	break;

    case uijkl:
	if ((test & (PRINT_STATS | RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS)) != 0)
	    printf ("uijkl:\n\r");
	active_edges = a + 2;
	denominator_form = 2;
	dv1[e + a + 1] = ni;
	dv2[e + a + 1] = nj;
	d0[e + a + 1] = branches + 1;
	branch[branches + 1][0] = 'f';
	branch[branches + 1][1] = '1';
	branch[branches + 1][2] = 0;
	fmap[e + a + 1] = 1;
	dv1[e + a + 2] = nk;
	dv2[e + a + 2] = nl;
	d0[e + a + 2] = branches + 2;
	branch[branches + 2][0] = 'f';
	branch[branches + 2][1] = '2';
	branch[branches + 2][2] = 0;
	fmap[e + a + 2] = 2;
	break;

    };

    for (i = 1; i <= (passive_edges + active_edges); i++) {
	di1[i] = dv1[i];
	di2[i] = dv2[i];
    };






    for (i = 1; i <= a; i++) {
	di1[e + i] = a3[i];
	di2[e + i] = a4[i];
    };

    switch (numerator) {

    case wijkl:
    case wijij:
    case wklkl:
    case wklij:
    {
    switch (denominator) {
    case V:
	break;

    case wijkl:
	di1[e + a + 1] = nk;
	di2[e + a + 1] = nl;
	break;

    case wijij:
	break;

    case wklkl:
	break;

    case wklij:
	di1[e + a + 1] = ni;
	di2[e + a + 1] = nj;
	break;

    case uijkl:
	break;

    };
    };
    break;
    case V:
    case uijkl:
    {
    switch (denominator) {
    case V:
	break;

    case wijkl:
	di1[e + a + 1] = (sign == 1) ? nk : nl;
	di2[e + a + 1] = (sign == 1) ? nl : nk;
	break;

    case wijij:
	if (sign == -1) {
	    j = di1[e + a + 1];
	    di1[e + a + 1] = di2[e + a + 1];
	    di2[e + a + 1] = j;
	};
	break;

    case wklkl:
	if (sign == -1) {
	    j = di1[e + a + 1];
	    di1[e + a + 1] = di2[e + a + 1];
	    di2[e + a + 1] = j;
	};
	break;

    case wklij:
	di1[e + a + 1] = (sign == 1) ? ni : nj;
	di2[e + a + 1] = (sign == 1) ? nj : ni;
	break;

    case uijkl:
	break;

    };

    };
    break;


    };



    if ((rc = product (minty_nodes, passive_edges + active_edges, actual_nodes,
		       denominator_form, e + a)) != 0) {
	printf ("insufficient memory to buffer denominator common trees; remainder of function evaluation skipped\n\r");
	return;
    };






    print_treebuf ((e + a));


}
 void 
print_treebuf (int nn)
{
    int s, i, ii, j, k, ch0, r, f, op, rc, tmptree[emax], term[nmax];


    if ((test & (PRINT_STATS)) != 0)
	printf ("opampcnt= %d\n\r", opampcnt);



    if ((test & (DEBUG_512)) == 0) {

        if(0 != fixup () ){
        printf("insufficient memory to complete processing\n");
        printf ("\n Numerator:\n\n");
	printf ("\n Denominator:\n\n");
        printf ("\n End: \n\n");
         release (NUM_VARS);
         release (DENOM_VARS);
         return;
        };

	printf ("\n Numerator:\n\n");

	numcntxt = 0;
	printexp (NUM_VARS);



	printf ("\n Denominator:\n\n");

	numcntxt = 1;
	printexp (DENOM_VARS);
    };
    release (NUM_VARS);
    release (DENOM_VARS);


    printf ("\n End: \n\n");

}
 void 
treecheck (int r, int *d1, int *d2, int *tree, int *perm)
{
    int stack[nmax], tag[nmax];
    int depth, i, j, done, root_node, princip_node;

    for (i = 1; i <= r; i++) {
	tag[i] = 0;
    };

    perm[0] = 0;
    perm[1] = 0;
    stack[0] = 1;
    depth = 0;
    for (done = 0; done == 0;) {
	root_node = stack[depth];
	for (i = 1; i <= r; i++) {
	    if (tag[i] == 1)
		continue;
	    if (d1[tree[i]] == root_node) {
		tag[i] = 1;
		princip_node = d2[tree[i]];
		perm[princip_node] = tree[i];
		break;
	    };
	    if (d2[tree[i]] == root_node) {
		tag[i] = 1;
		princip_node = d1[tree[i]];
		perm[princip_node] = (-tree[i]);
		break;
	    };
	};
	if (i <= r) {
	    depth++;
	    stack[depth] = princip_node;
	} else {
	    if (depth > 0)
		depth--;
	    else
		done = 1;
	};
    };
    return;
}
 int 
product (int minty_nodes, int edges, int actual_nodes, int form, int br)
{
    int i, ii, j, k, m, nn, ncom, s, rc, sign, tmp, itreepermbuf[nmax], vtreepermbuf[nmax];
    int tree[nmax];
    int op, ch0;

#define maxrunlength 10000
    int run, runstats[maxrunlength];

    for (run = 0; run < maxrunlength; run++)
	runstats[run] = 0;

    ncom = 0;

    istate = 0;
    vstate = 0;

    mn = minty_nodes;
    me = edges;
    mr = actual_nodes;
    mq = form;

    for (;;) {

/* search for common trees only */

	vminty ();
	iminty ();

	{
	    m = 0;
	    run = 0;
	    for (; (istate == 1) && (vstate == 1) && (m == 0);) {
		for (nn = 1; nn < r; nn++) {
		    if (itree[nn] > vtree[nn]) {
			vminty ();
			m = 0;
			run++;
			break;
		    } else {
			if (itree[nn] < vtree[nn]) {
			    iminty ();
			    m = 0;
			    run++;
			    break;
			} else
			    m = 1;
		    };
		};
	    };
	    if (m == 0) {
    if ((test & (PRINT_STATS)) != 0) {
    printf("raw i-trees  evaluated:%9d\n\r",i_nt);
    printf("raw v-trees  evaluated:%9d\n\r",vnt);
    printf("qualified common trees:%9d\n\r",ncom);
/*
    printf ("run statistics in matching v- and i- trees: number of trees evaluated and compared before matching\n\r");
    for (run = 0; run < maxrunlength; run++)
	if (runstats[run] != 0)
	    printf ("runlength= %5d, count= %d\n\r", run, runstats[run]);
*/
    };
		return (0);
	    };

	    if (run >= maxrunlength)
		run = maxrunlength - 1;
	    runstats[run]++;

            ncom++;

	    treecheck (r - 1, di1, di2, itree, &itreepermbuf[0]);
	    treecheck (r - 1, dv1, dv2, vtree, &vtreepermbuf[0]);

	    if ((test & (TREE_PERM)) != 0) {
		printf (" common tree edges:");
		for (i = 1; i < r; i++)
		    printf (" %3d ", itree[i]);
		printf ("\n");

		printf ("v-tree permutation: ");
		for (i = 0; i <= r; i++) {
		    printf ("%3d ", vtreepermbuf[i]);
		};
		printf ("\n");
		printf ("i-tree permutation: ");
		for (i = 0; i <= r; i++) {
		    printf ("%3d ", itreepermbuf[i]);
		};
		printf ("\n");
	    }
/* capture the principal node component of the sign computation */ sign = 1;
	    for (i = 2; i <= r; i++) {
		if (vtreepermbuf[i] < 0) {
		    sign = sign * (-1);
		    vtreepermbuf[i] = (-vtreepermbuf[i]);
		};
		if (itreepermbuf[i] < 0) {
		    sign = sign * (-1);
		    itreepermbuf[i] = (-itreepermbuf[i]);
		};
	    };
/* capture the permutation component of the sign computation */
	    for (i = 2; i <= r; i++) {
		{
		    if (vtreepermbuf[i] != itreepermbuf[i]) {
			sign = sign * (-1);
			for (j = i + 1; j <= r; j++) {
			    if (vtreepermbuf[j] == itreepermbuf[i]) {
				tmp = vtreepermbuf[j];
				vtreepermbuf[j] = vtreepermbuf[i];
				vtreepermbuf[i] = tmp;
				break;
			    };
			};
		    };
		};
	    };
	    if ((test & (TREE_PERM)) != 0)
		printf ("resulting sign= %2d\n", sign);
	};

	s = 0;
	for (i = 1, ii = 0, op = 0; i < r; i++) {
	    ch0 = branch[d0[itree[i]]][0];
	    s = s + powermap[itree[i]];
	    if ((ch0 == 'o') ||
		(ch0 == 'O'))
		op++;
	    if (fmap[itree[i]] != 0)
		continue;
	    tree[ii] = itree[i];
	    ii++;
	};
	if (opampcnt < op)
	    opampcnt = op;

	if (1 == dtag) {
	    if ((rc = getexpr (sign, s, ii, tree, br, DENOM_VARS)) != 0) {
		printf ("insufficient memory for combining admittance terms\n\r");
    printf("raw i-trees  evaluated:%9d\n\r",i_nt);
    printf("raw v-trees  evaluated:%9d\n\r",vnt);
    printf("qualified common trees:%9d\n\r",ncom);
		release (NUM_VARS);
		release (DENOM_VARS);


		printf ("\n End: \n\n");

		return (rc);
	    };
	} else if ((rc = getexpr (sign, s, ii, tree, br, NUM_VARS)) != 0) {
	    printf ("insufficient memory for combining admittance terms\n\r");
    printf("raw i-trees  evaluated:%9d\n\r",i_nt);
    printf("raw v-trees  evaluated:%9d\n\r",vnt);
    printf("qualified common trees:%9d\n\r",ncom);
	    release (NUM_VARS);
	    release (DENOM_VARS);


	    printf ("\n End: \n\n");

	    return (rc);
	};

    };


    return (0);
}


 void 
iminty (void)
{				/* avoid parameter passing if possible */
    int vd1, vd2, i, j, k, l, m, f0, f1, f2;
    int v[nmax], edge[emax];

    switch (istate) {
    case 0:
	goto l10;
    case 1:
	goto l25;
    default:
	istate = 2;
	return;
    };
  l10:
    istate = 1;

    if ((test & (RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS | DEBUG_128)) != 0) {
        printf("only %s-trees are accepted\n",(mq == 0)?"2":((mq == 1)?"1":"3"));
        printf ("i-graph to generate i-trees:\n\rname:edge#,nameindex,node1,#node2#\n\r");
	for (i = 1; i <= me; i++)
	    printf ("%5s: d0[%3d]=%3d, d1=%3d, d2=%3d\n\r", &branch[d0[i]][0], i, d0[i], di1[i], di2[i]);
	printf ("\n\r\n\r");
    };

    for (k = 1; k <= me; k++)
	edge[k] = 1;
    for (k = 1; k <= mn; k++)
	v[k] = 0;
    iit = 0;
    i = 1;
    i_nt = 0;
  l15:
    for (k = 1; (k <= me); k++)
	if (edge[k] == 1)
	    goto l40;
  l25:
    if (iit <= 1) {
	istate = 2;
	return;
    };
    (iit)--;
    for (k = 1; k <= me; k++)
	edge[k] = iestk[iit][k];
    for (k = 1, i = 0; k <= mn; k++) {
	v[k] = ivstk[iit][k];
	if (v[k] > i)
	    i = v[k];
    };
    i++;
    goto l15;

  l40:
    l = k;
    edge[l] = 0;
    m = 0;
    for (k = 1; k <= me; k++)
	if (edge[k] != 0)
	    m++;
    if (m >= (mn - 1)) {

	for (k = 1; k <= me; k++)
	    iestk[iit][k] = edge[k];
	for (k = 1; k <= mn; k++)
	    ivstk[iit][k] = v[k];
	(iit)++;
    }
  l60:
    edge[l] = -1;
    vd1 = di1[l];
    vd2 = di2[l];

    if (v[vd1] == 0)
	goto l95;
    if (v[vd2] == 0)
	goto l105;

    l = v[vd1];
    m = v[vd2];

    if (l == m)
	goto l25;
    if (l > m) {
	k = m;
	m = l;
	l = k;
    };
    for (k = 1; k <= mn; k++) {
	if (v[k] > m)
	    v[k]--;
	else if (v[k] == m)
	    v[k] = l;
    };
    i--;
    goto l110;
  l95:
    if (v[vd2] == 0)
	goto l100;
    v[vd1] = v[vd2];
    goto l110;
  l100:
    v[vd1] = i;
    v[vd2] = i;
    i++;
    goto l15;
  l105:
    v[vd2] = v[vd1];
  l110:
    for (k = 1; k <= mn; k++)
	if (v[k] != 1)
	    goto l15;
    (i_nt)++;
    l = 1;

    if ((test & (RAW_TREES)) != 0) {
        printf (" raw i-tree no. %5d edges:", i_nt);
        for (k = 2; k <= me; k++)
            if(edge[k] == -1) printf (" %3d ", k);
    };


/* q == 0 rejects all trees without 'f' branch; q == 1 rejects all trees with 'f' branch;
   q == 2 rejects all trees without both 'f1' and 'f2 branches */

    f0 = 0;
    f1 = 0;
    f2 = 0;
    for (k = 2; k <= me; k++)	/* dummy edge 1 is always present, so skip it */
	if (edge[k] == -1) {
	    itree[l] = k;
	    l++;
	    switch (fmap[k]) {
	    case 0:
		break;
	    case 1:
		f1 = 1;
		break;
	    case 2:
		f2 = 1;
		break;
	    case 3:
                if (mq == 1){
                  if ((test & (RAW_TREES)) != 0)
                      printf(" rejected: not a 1-tree\n");
                  goto l25;
                };
                     f0 = 1;
		break;
	    default:;
	    };

	};
    if (((mq == 0) || (mq == 1)) && (mq == f0))
    {
        if ((test & (RAW_TREES)) != 0) {
                    if(mq == 1)printf(" rejected: not a 1-tree\n");
                    else printf(" rejected: not a 2-tree\n");
             };
        goto l25;
    };
    if ((mq == 2) && ((f1 == 0) || (f2 == 0)))
    {
        if ((test & (RAW_TREES)) != 0)
                  printf(" rejected: not a 3-tree\n");
        goto l25;

    };

    if ((test & (RAW_TREES)) != 0) printf(" accepted\n");

    return;

}
 void 
vminty (void)
{
    int vd1, vd2, i, j, k, l, m, f0, f1, f2;
    int v[nmax], edge[emax];

    switch (vstate) {
    case 0:
	goto l10;
    case 1:
	goto l25;
    default:
	vstate = 2;
	return;
    };
  l10:
    vstate = 1;

    if ((test & (RAW_TREES | BUFF_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS | DEBUG_128)) != 0) {
        printf("only %s-trees are accepted\n",(mq == 0)?"2":((mq == 1)?"1":"3"));
	printf ("v-graph to generate v-trees:\n\rname:edge#,nameindex,node1,#node2#\n\r");
	for (i = 1; i <= me; i++)
	    printf ("%5s: d0[%3d]=%3d, d1=%3d, d2=%3d\n\r", &branch[d0[i]][0], i, d0[i], dv1[i], dv2[i]);
	printf ("\n\r\n\r");
    };

    for (k = 1; k <= me; k++)
	edge[k] = 1;
    for (k = 1; k <= mn; k++)
	v[k] = 0;
    vit = 0;
    i = 1;
    vnt = 0;
  l15:
    for (k = 1; (k <= me); k++)
	if (edge[k] == 1)
	    goto l40;
  l25:
    if (vit <= 1) {

	vstate = 2;
	return;
    };
    (vit)--;
    for (k = 1; k <= me; k++)
	edge[k] = vestk[vit][k];
    for (k = 1, i = 0; k <= mn; k++) {
	v[k] = vvstk[vit][k];
	if (v[k] > i)
	    i = v[k];
    };
    i++;
    goto l15;

  l40:
    l = k;
    edge[l] = 0;
    m = 0;
    for (k = 1; k <= me; k++)
	if (edge[k] != 0)
	    m++;
    if (m >= (mn - 1)) {

	for (k = 1; k <= me; k++)
	    vestk[vit][k] = edge[k];
	for (k = 1; k <= mn; k++)
	    vvstk[vit][k] = v[k];
	(vit)++;
    }
  l60:
    edge[l] = -1;
    vd1 = dv1[l];
    vd2 = dv2[l];

    if (v[vd1] == 0)
	goto l95;
    if (v[vd2] == 0)
	goto l105;

    l = v[vd1];
    m = v[vd2];

    if (l == m)
	goto l25;
    if (l > m) {
	k = m;
	m = l;
	l = k;
    };
    for (k = 1; k <= mn; k++) {
	if (v[k] > m)
	    v[k]--;
	else if (v[k] == m)
	    v[k] = l;
    };
    i--;
    goto l110;
  l95:
    if (v[vd2] == 0)
	goto l100;
    v[vd1] = v[vd2];
    goto l110;
  l100:
    v[vd1] = i;
    v[vd2] = i;
    i++;
    goto l15;
  l105:
    v[vd2] = v[vd1];
  l110:
    for (k = 1; k <= mn; k++)
	if (v[k] != 1)
	    goto l15;
    (vnt)++;
    l = 1;

    if ((test & (RAW_TREES)) != 0) {
        printf (" raw v-tree no. %5d edges:", vnt);
        for (k = 2; k <= me; k++)
            if(edge[k] == -1) printf (" %3d ", k);
    };

/* q == 0 rejects all trees without 'f' branch; q == 1 rejects all trees with 'f' branch;
   q == 2 rejects all trees without both 'f1' and 'f2 branches */

    f0 = 0;
    f1 = 0;
    f2 = 0;
    for (k = 2; k <= me; k++)	/* dummy edge 1 is always present, so skip it */
	if (edge[k] == -1) {
	    vtree[l] = k;
	    l++;
	    switch (fmap[k]) {
	    case 0:
		break;
	    case 1:
		f1 = 1;
		break;
	    case 2:
		f2 = 1;
		break;
	    case 3:
                if (mq == 1){
                  if ((test & (RAW_TREES)) != 0)
                      printf(" rejected: not a 1-tree\n");
                  goto l25;
                };
		f0 = 1;
		break;
	    default:;
	    };

	};


    if (((mq == 0) || (mq == 1)) && (mq == f0))
    {
        if ((test & (RAW_TREES)) != 0) {
                    if(mq == 1)printf(" rejected: not a 1-tree\n");
                    else printf(" rejected: not a 2-tree\n");
             };
        goto l25;
    };
    if ((mq == 2) && ((f1 == 0) || (f2 == 0)))
    {
        if ((test & (RAW_TREES)) != 0)
                  printf(" rejected: not a 3-tree\n");
        goto l25;

    };


    if ((test & (RAW_TREES)) != 0) printf(" accepted\n");

    return;

}
 /* code to combine like terms */ void 
combineinit (void)
{
    int i, j, k, ch0;

    init (NUM_VARS);
    init (DENOM_VARS);

    for (i = 1, opamp = 0; i <= (e + a); i++) {

	if ((branch[d0[i]][0] == 'o') ||
	    (branch[d0[i]][0] == 'O')) {
	    opamp = i;
	    break;
	};
    };

    for (j = 1; j <= (e + a); j++) {
	branchmap[j] = getsym (j, (e + a));
    };

    for (i = 0; i < maxsym; i++)
	branchtag[i] = 0;

    impedancecount = 0;

    for (i = 1; i <= (e + a); i++) {


	j = getsym (i, (e + a));
	ch0 = branch[d0[j]][0];

	if ((ch0 == 'c') ||
	    (ch0 == 'C') ||
	    (ch0 == 'l') ||
	    (ch0 == 'L') ||
	    (ch0 == 'm') ||
	    (ch0 == 'M'))
	    powermap[i] = 1;
	else
	    powermap[i] = 0;

	if (ch0 == 'a')
	    fmap[i] = 4;
	else
	    fmap[i] = 0;

	if (i > j)
	    continue;
	if ((ch0 == 'r') ||
	    (ch0 == 'R') ||
	    (ch0 == 'z') ||
	    (ch0 == 'Z') ||
	    (ch0 == 'l') ||
	    (ch0 == 'L') ||
	    (ch0 == 'm') ||
	    (ch0 == 'M')) {
	    impedancelist[impedancecount] = j;
	    impedancecount++;
	};
    };


}
 void 
init (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly)
{
    int i, j, k;
    int (*poly)[maxdegree], (*termlist)[maxsym + 2];

    *maxtermlist = maxterm * maxdegree;
    *maxpoly = maxterm;
    *ptermlist = (void *) malloc ((*maxtermlist) * sizeof (int) * (maxsym + 2));
    *ppoly = (void *) malloc ((*maxpoly) * sizeof (int) * maxdegree);

    poly = *ppoly;
    termlist = *ptermlist;

    for (j = 0; j < maxdegree; j++)
	for (k = 0; k < (*maxpoly); k++)
	    poly[k][j] = 0;

    for (i = 0; i < (*maxtermlist); i++)
	for (j = 0; j <= (maxsym + 1); j++)
	    termlist[i][j] = 0;
    for (j = 0; j < maxdegree; j++)
	degterms[j] = 0;
    *toterms = 0;
    termlistoverage = 0;
}
int resize(void ** ptr, int * count, int module)
{
  int tmpsize;
  void * tmpptr;
                printf ("orig size =%d, orig ptr =0x%08x\n\r",(*count) ,(*ptr) );
                tmpsize = 2 * (*count);
                tmpptr = (void *) realloc ((*ptr), tmpsize * module);

		if ((tmpptr != (void *) 0)) {
                    printf ("num of elements =%d, pointer =0x%08x\n\r", tmpsize, tmpptr);
                    *ptr = tmpptr;
                    *count = tmpsize;
                    return(0);
		} else {
                    tmpsize = (tmpsize + (*count)) / 2;
                    while (tmpsize > (*count)) {
                        tmpptr = (void *) realloc ((*ptr), tmpsize * module);

			if ((tmpptr != (void *) 0)) {
                            printf ("num of elements =%d, pointer =0x%08x\n\r", tmpsize, tmpptr);
                            *ptr = tmpptr;
                            *count = tmpsize;
                            return(0);
			} else {
                            tmpsize = (tmpsize + (*count)) / 2;
                            if (tmpsize == (*count)){
                                printf("resize failed for insufficient memory\n");
				return (1);
                            };
			};
		    }
		};
                printf("resize failed for insufficient memory\n");
                return(1);

}

 void
release (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly)
{
    free (*ptermlist);
    free (*ppoly);

}
 void 
eval_branchtag (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly)
{
/* branchtag collects all the impedance-unit factors to be divided out later */

    int i, j, k, l, m, n;
    int (*poly)[maxdegree], (*termlist)[maxsym + 2];


    poly = *ppoly;
    termlist = *ptermlist;


    if ((test & (DEBUG_512)) != 0)
	return;


    for (i = 0, n = 0; i < maxdegree; i++) {
	if (degterms[i] > 0) {
	    for (j = 0; (j < degterms[i]); j++) {
		l = poly[j][i];
		if ((termlist[l][maxsym] != 0)) {
		    if ((termlist[l][opamp] == opampcnt)) {
			for (k = 0; k < impedancecount; k++) {
			    m = impedancelist[k];
			    if (termlist[l][m] > branchtag[m])
				branchtag[m] = termlist[l][m];
			};
		    } else {
			termlist[l][maxsym] = 0;
			if ((j + 1) < degterms[i])
			    for (m = (j + 1); m < degterms[i]; m++)
				poly[m - 1][i] = poly[m][i];
			degterms[i]--;
			j--;

		    };

		};

	    };
	};
    };



    return;
}
 int 
getexpr (int sign, int deg, int factors, int *factorlist, int edges, int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly)
{
    int i, j, k, l, m, n;
    int u,v;
    int (*poly)[maxdegree], (*termlist)[maxsym + 2];
    int tmpsize,origsize;
    void *tmpptr;

    poly = *ppoly;
    termlist = *ptermlist;

    for (i = 0; i < maxsym; i++)
	term[i] = 0;

    for (j = 0; j < factors; j++) {
	/* i = getsym(factorlist[j], edges); */
	i = branchmap[factorlist[j]];
	term[i] = term[i] + 1;
    };
/*debug int denfn,numfn,inptno,evfn; */
    if ((test & (DEBUG_256)) != 0) {
	printf ("%2d,%2d,%2d,%2d,%2d,%2d:", inptno, evfn, numfn, denfn, deg, sign);
	for (j = 0; j < factors; j++)
	    printf (",%3d", factorlist[j]);
	printf ("\n");
    };
    if ((test & (DEBUG_512)) != 0)
	return (0);

    if (degterms[deg] == 0) {
	poly[0][deg] = *toterms;
	for (i = 0; i < maxsym; i++)
	    termlist[*toterms][i] = term[i];
	termlist[*toterms][maxsym] = sign;
	termlist[*toterms][maxsym + 1] = 0;
        if (((*toterms)+1) >= (*maxtermlist)) {
	    {
                printf("resizing termlist ");
                tmpsize = (*maxtermlist);
                origsize = tmpsize;
                tmpptr = (*ptermlist);
                if(0 != resize((void *)&tmpptr, &tmpsize, sizeof (int) * (maxsym + 2)) ){
                 return(1);
                 };
		    *ptermlist = tmpptr;
		    *maxtermlist = tmpsize;
		    termlist = *ptermlist;
            } for (i = origsize; i < (*maxtermlist); i++)
		for (j = 0; j <= (maxsym + 1); j++)
		    termlist[i][j] = 0;

	}
	(*toterms)++;
	degterms[deg] = 1;

    } else {
/*        for (m = 0, n = 0; (m < degterms[deg]) && (n == 0); m++) {
	    k = poly[m][deg];
            for (i = 0; i < maxsym; i++)
		if (termlist[k][i] != term[i])
		    break;
	    if (i == maxsym) {
		n = 1;
		termlist[k][maxsym] += sign;

	    };
	};
*/

        for (m = (degterms[deg]-1)/2, u = 0, v = (degterms[deg]-1),n = 0; (u <= v) && (n >= 0); m = (u+v)/2) {
/*printf("m,u,v,deg: %5d, %5d, %5d, %2d\n\r",m,u,v,deg);*/

	    k = poly[m][deg];
            for (i = 0; i < maxsym; i++){
                if (termlist[k][i] < term[i]){
                    v = m-1;
                    n = m;
		    break;
                    }
                else
                if (termlist[k][i] > term[i]){
                    u = m+1;
                    n = m+1;
		    break;
                    }
                else
                n = -1;
            };
            if (n < 0) {
		termlist[k][maxsym] += sign;

	    };
	};
        if (n >= 0) {
	    k = degterms[deg];
            if(n < k)
               for(i=k; i>n; i--)
               poly[i][deg] = poly[i-1][deg];
               
            poly[n][deg] = *toterms;
	    for (i = 0; i < maxsym; i++)
		termlist[*toterms][i] = term[i];
	    termlist[*toterms][maxsym] = sign;
	    termlist[*toterms][maxsym + 1] = 0;

            if (((*toterms)+1) >= (*maxtermlist)) {
		{

                printf("resizing termlist ");
                tmpsize = (*maxtermlist);
                origsize = tmpsize;
                tmpptr = (*ptermlist);
                if(0 != resize((void *)&tmpptr, &tmpsize, sizeof (int) * (maxsym + 2)) ){
                 return(1);
                 };
		    *ptermlist = tmpptr;
		    *maxtermlist = tmpsize;
		    termlist = *ptermlist;

                } for (i = origsize; i < (*maxtermlist); i++)
		    for (j = 0; j <= (maxsym + 1); j++)
			termlist[i][j] = 0;

	    }
	    (*toterms)++;
	    degterms[deg]++;
            if ((degterms[deg]+1) >= (*maxpoly)) {
		{

                printf("resizing poly ");
                    tmpsize = (*maxpoly);
                    origsize = tmpsize;
                    tmpptr = (*ppoly);

                if(0 != resize((void *)&tmpptr, &tmpsize, sizeof (int) * maxdegree) ){
                 return(1);
                 };
			*ppoly = tmpptr;
			*maxpoly = tmpsize;
		    poly = *ppoly;
		} for (j = 0; j < maxdegree; j++)
                    for (k = origsize; k < (*maxpoly); k++)
			poly[k][j] = 0;


	    };


	};
    };

    return (0);
}
 void 
printexp (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly)
{
    int i, j, k, l, m, n;

    int (*poly)[maxdegree], (*termlist)[maxsym + 2];

    poly = *ppoly;
    termlist = *ptermlist;


    if (termlistoverage != 0)
	printf ("*******termlist size exceeded by %d terms\n\r", termlistoverage);

    if (cancelcoeff == 0)
	cancelcoeff = 1;

    if ((test & (ADMITT_UNITS)) != 0) {
	printf ("common factors to cancel in all terms of both num and denom:\n\r");
	printf ("%s%d ", cancelsign > 0 ? "+" : "-", cancelcoeff);
	for (k = 0; k < maxsym; k++)
	    if ((cancelfactor[k]) > 0) {
		if ((cancelfactor[k]) > 1)
		    printf (" %s^%-2d", &branch[d0[k]][0], (cancelfactor[k]));
		else
		    printf (" %s   ", &branch[d0[k]][0]);
	    };

	printf ("\n\r");
    };



    for (i = 0, n = 0; i < maxdegree; i++) {
	if (degterms[i] > 0) {
	    for (j = 0, k = 0; (j < degterms[i]) && (k == 0); j++) {
		l = poly[j][i];
		if ((termlist[l][maxsym] != 0))
		    k = 1;

	    };

	    if (k == 1) {
		n++;
		printf ("\n\n + s^%-2d (\n", i - cancelexp);
		for (j = 0; j < degterms[i]; j++) {
		    l = poly[j][i];
		    if ((termlist[l][maxsym] == 0))
			continue;
		    if ((test & (DEBUG_1024)) != 0)
			printf ("%2d,%2d,%2d,%2d,%2d:", inptno, evfn, numcntxt, i - cancelexp, (((cancelsign * termlist[l][maxsym] / cancelcoeff) < 0) ? -1 : 1));
		    else
			printf ("         ");
		    {
			if (((cancelsign * termlist[l][maxsym] / cancelcoeff) > 1) ||
			    ((cancelsign * termlist[l][maxsym] / cancelcoeff) < 0))
			    printf ("%3d", cancelsign * termlist[l][maxsym] / cancelcoeff);
			else {
			    if ((test & (DEBUG_1024)) != 0)
				printf ("  1");
			    else
				printf ("   ");
			};
		    };
		    for (k = 0, m = 0; k < maxsym; k++) {
			if ((termlist[l][k] - cancelfactor[k]) > 0) {
			    m = 1;
			    if ((termlist[l][k] - cancelfactor[k]) > 1)
				printf (" %s^%-2d", &branch[d0[k]][0], (termlist[l][k] - cancelfactor[k]));
			    else
				printf (" %s   ", &branch[d0[k]][0]);
			};
		    };
		    if ((m == 0) && ((cancelsign * termlist[l][maxsym] / cancelcoeff) == 1))
			printf (" 1");
		    /*if(termlist[l][maxsym+1]==0)printf(" missed"); */ printf ("\n");
		};
		printf ("        )");
	    };
	};
    };
    if (n == 0)
	printf ("        0");
    printf ("\n");
}
 int testbranchtag[emax];
int 
fixup (void)
{
    int j, k;


    eval_branchtag (NUM_VARS);
    eval_branchtag (DENOM_VARS);

    for (j = 1; j <= (e + a); j++) {
	testbranchtag[j] = 0;
	if (branchtag[j] == 0)
	    continue;
	else {
	    k = getsym (j, (e + a));
	    if (j > k) {
		branchtag[j] = 0;
		branchtag[k]++;
	    };
	};
    };
    if(0 != correctunits (NUM_VARS) )
      return(1);
    if(0 != correctunits (DENOM_VARS) )
      return(1);
    makecancellationlist (NUM_VARS);
    makecancellationlist (DENOM_VARS);

    return(0);

}
 int 
correctunits (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly)
{
    int i, ii, j, k, l, m, n, s;
    int kk;
    int u,v;
    int (*poly)[maxdegree], (*termlist)[maxsym + 2];
    int tmpsize, origsize;
    void *tmpptr;


    poly = *ppoly;
    termlist = *ptermlist;



    for (i = 0; i < maxdegree; i++) {
	if (degterms[i] > 0) {
	    for (j = 0, k = 0; (j < degterms[i]) && (k == 0); j++) {
		l = poly[j][i];
		if (		/*(termlist[l][maxsym] != 0)&& */
		       (termlist[l][maxsym + 1] == 0))
		    k = 1;

	    };

	    if (k == 1) {
		for (j = 0; j < degterms[i]; j++) {
		    l = poly[j][i];
		    if (	/*(termlist[l][maxsym] == 0) || */
			   (termlist[l][maxsym + 1] == 1))
			continue;
		    termlist[l][maxsym + 1] = 1;



		    for (k = 0; k < maxsym; k++) {
			if ((branchtag[k] != 0)) {
			    termlist[l][k] = branchtag[k] - termlist[l][k];
			    /*termlist[l][maxsym+1] = 1; */
			};
		    };
		    for (k = 0, s = 0; k < maxsym; k++) {
			if (termlist[l][k] != 0) {
			    m = branch[d0[k]][0];
			    if ((m == 'c') ||
				(m == 'C') ||
				(m == 'l') ||
				(m == 'L') ||
				(m == 'm') ||
				(m == 'M'))
				s += termlist[l][k];
			};
		    };
		    if (s != i) {
			if (degterms[s] == 0) {
			    poly[0][s] = l;
			    degterms[s] = 1;

			} else {
			    for (m = 0, n = 0; (m < degterms[s]) && (n == 0); m++) {
				k = poly[m][s];
				if (termlist[k][maxsym + 1] == 0)
				    continue;
				for (ii = 0; ii < maxsym; ii++)
				    if (termlist[k][ii] != termlist[l][ii])
					break;
				if (ii == maxsym) {
				    n = 1;
				    termlist[k][maxsym] += termlist[l][maxsym];

				};
			    };
			    if (n == 0) {
				k = degterms[s];
                                if ((degterms[s]+1) >= (*maxpoly)) {
				    {





                printf("resizing poly ");
                    tmpsize = (*maxpoly);
                    origsize = tmpsize;
                    tmpptr = (*ppoly);

                if(0 != resize((void *)&tmpptr, &tmpsize, sizeof (int) * maxdegree) ){
                 return(1);
                 };
			*ppoly = tmpptr;
			*maxpoly = tmpsize;
		    poly = *ppoly;
                } for (ii = 0; ii < maxdegree; ii++)
                    for (n = origsize; n < (*maxpoly); n++)
                        poly[n][ii] = 0;

				};
				poly[k][s] = l;
				degterms[s]++;


			    };
			};
			if ((j + 1) < degterms[i])
			    for (ii = (j + 1); ii < degterms[i]; ii++)
				poly[ii - 1][i] = poly[ii][i];
			degterms[i]--;
			j--;
		    };
		};
	    };
	};
    };




    for (i = 0; i < maxdegree; i++) {
	if (degterms[i] > 0) {
	    for (j = (degterms[i] - 1); (j >= 0); j--) {
		l = poly[j][i];
		if (		/*(termlist[l][maxsym] != 0)&& */
		       (termlist[l][maxsym + 1] == 1)) {

/*********/
for(v=0,u=(j-1),k=((j-1)/2);(u>=v);k=((u+v)/2)){
    m = poly[k][i];

                        for (ii = 0; ii < maxsym; ii++){
                            if (termlist[l][ii] < termlist[m][ii])
                                {
                                  v = k + 1;
                                  break;
                                }
                             else
                            if (termlist[l][ii] > termlist[m][ii])
                                {
                                  u = k - 1;
                                  break;
                                };

                        };

                        if (ii == maxsym) {

                            termlist[m][maxsym] += termlist[l][maxsym];
                            if ((j + 1) < degterms[i])
                                for (ii = (j + 1); ii < degterms[i]; ii++)
                                    poly[ii - 1][i] = poly[ii][i];
                            degterms[i]--;

                            break;
                        };




};

/*********/

#if 0
		    for (k = 0; k < j; k++) {
			m = poly[k][i];
			if (	/*(termlist[m][maxsym] == 0) || */
			       (termlist[m][maxsym + 1] == 0))
			    continue;


			for (ii = 0; ii < maxsym; ii++)
			    if (termlist[l][ii] != termlist[m][ii])
				break;
			if (ii == maxsym) {

			    termlist[m][maxsym] += termlist[l][maxsym];
			    if ((j + 1) < degterms[i])
				for (ii = (j + 1); ii < degterms[i]; ii++)
				    poly[ii - 1][i] = poly[ii][i];
			    degterms[i]--;

			};




		    };
#endif



		};

	    };
	};
    };

    return(0);


}
 void 
makecancellationlist (int (**ptermlist)[maxsym + 2], int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly)
{
    int i, j, k, l, m, n;

    int (*poly)[maxdegree], (*termlist)[maxsym + 2];


    poly = *ppoly;
    termlist = *ptermlist;

    for (i = 0; i < maxdegree; i++) {
	if (degterms[i] > 0) {
	    for (j = 0, k = 0; (j < degterms[i]) && (k == 0); j++) {
		l = poly[j][i];
		if ((termlist[l][maxsym] != 0))
		    k = 1;

	    };

	    if (k == 1) {
		if (cancelexp > i)
		    cancelexp = i;
		for (j = 0; j < degterms[i]; j++) {
		    l = poly[j][i];
		    if ((termlist[l][maxsym] == 0))
			continue;

		    for (k = 0; k < maxsym; k++) {
			if ((termlist[l][k] < cancelfactor[k]))
			    cancelfactor[k] = termlist[l][k];
		    };

		    if ((termlist[l][maxsym] > 0))
			cancelsign = 1;

		    if (((n = termlist[l][maxsym]) < 0))
			n = -n;
		    if ((cancelcoeff > 0) && (cancelcoeff != n)) {
			if (cancelcoeff > n) {
			    m = n;
			    n = cancelcoeff;
			} else
			    m = cancelcoeff;

			for (; n > m;) {
			    for (; n > m;)
				n -= m;
			    l = m;
			    m = n;
			    n = l;
			};
			cancelcoeff = n;
		    } else
			cancelcoeff = n;
		};
	    };
	};
    };
}
 int 
getsym (int i, int edges)
{

    int j, k, l, m;

    switch (branch[d0[i]][0]) {
    case 'o':
    case 'O':
	return (opamp);
    default:;
    };

    k = 1;
    while (k <= edges) {
	m = 0;
	while ((branch[d0[k]][m] == branch[d0[i]][m]) && (branch[d0[i]][m] != 0))
	    m++;
	if (branch[d0[k]][m] == branch[d0[i]][m]) {
	    return k;
	};
	k++;
    };
    printf ("can't find %s <=> branch[%d] in %d branches\n\r", &branch[d0[i]][0], d0[i], edges);
    return i;

}
