/* tree45a.c (derived from a4)
adds optional component values to weight the products.
maeyeda's 2-graph method to evaluate topological formulas using
(minty's algorithm for spanning tree enumeration
in symbolic network analysis, lin) */
#define gcc 0
#if gcc
#include <stdio.h>
#include <time.h>
#else
#include "..\..\include\time.h"
#include "..\..\include\stdio.h"
#endif

char * copyright_notice[] ={
"\n\nThis software is copyright by Jeffrey Furman 2000-2004.\n",
"It may be freely used and copies may be given to other\n",
"users as long as the following conditions are met:\n",
"1. This licensing agreement must accompany any copies which are distributed.\n",
"2. No charge may be made for the copies.\n",
"3. All applications and usage of the program must be non-commercial.\n",
"It is provided STRICTLY without warranty, expressed or implied. The\n",
"author and the distributor(s) shall not be liable for any damages,\n",
"special, indirect, incidental, consequential or otherwise resulting\n",
"from the use of this software. No warranty is provided on\n",
"the correctness of any results obtained from using the software.\n",
};

/* this is an experimental change, not implemented now
see Lin, section 3.4.5, pp 81-84*/
/*#define  COMBINED_TREES*/

#define ABS(x) (((x) >= 0)?(x):(-(x)))

/*resource burden for s and F parameter embedding network*/
#define s_active (4 + 2) /* +2 for link(s) for optional normalizing networks*/
#define s_passive 8
#define s_nodes 8

/* change the constants here to change limits for resources */
#define actmax (30+s_active)               /* max active 2-ports */
#define pmax (60+s_passive)                 /*max passive (one-port) edges */
#define emax (pmax+actmax)	/* max edges */
#define nmax (35+s_nodes)                 /* max nodes */
#define stk1 nmax		/* minty alg. stack */

#define BRLEN 12 /*max chars in branch names*/

/* numerator and denominator indicators;
  used as indexes for vars[] */
#define NVARS 10 /* 0..9 for this list of indices */
#define V     0
#define wijkl 1
#define wijij 2
#define wklkl 3
#define wklij 4
#define uijkl 5
#define saved_uijkl 6
#define saved_wijkl 7
#define numerator 8
#define denominator 9

/* test parameter masks */
#define INPUT_DUMP 1
#define RAW_TREES 2
#define PRINT_TRACE 4
#define TREE_PERM 8
#define MIXED_UNITS 16
#define ADMITT_UNITS 32
#define PRINT_STATS 64
#define DEBUG_128 128
#define DEBUG_256 256
#define DEBUG_512 512
#define DEBUG_1024 1024
#define DEBUG_2048 2048

/* tree element type flags */
#define OP 1
#define F1 2
#define F2 4
/* this doesn't work properly, so, it's set to 0 here, otherwise it would be 8*/
#define HANDLE_BRANCH 0
#define ONETREES 0
#define TWOTREES F1
#define THREETREES (F1 | F2)
#define TREEMASK (F1 | F2)
#define UNITY 16
#ifdef COMBINED_TREES
#define ESSENTIAL_BRANCH_MASK (OP | HANDLE_BRANCH)
#else
#define ESSENTIAL_BRANCH_MASK (OP | F1 | F2 | HANDLE_BRANCH)
#endif

int denfn, numfn, inptno, evfn, numcntxt;

int nbuf, nbuf1;
int branchtag[emax], impedancelist[emax], impedancecount;
int powermap[emax], dpowermap[emax], fmap[emax], dmap[emax];
int dtag;
int ni, nj, nk, nl, ni1, nj1, nk1, nl1;
int normalization, normalnet1, normalnet2, normalnet3, normalnet4;  
int opampcnt, opamp, n, int_node, nodelist[nmax];
int oprej;
int p0[pmax], p1[pmax], p2[pmax];
int test;

int e, p, r, a, d0[emax], di1[emax], di2[emax], dv1[emax], dv2[emax];
int a0[actmax], a1[actmax], a2[actmax], a3[actmax], a4[actmax];
volatile int branches;
char branch[emax][BRLEN+1];
double value[emax];
char ch = 0;

#define maxsym emax
#define maxterm emax
#define maxdegree nmax

typedef struct vardef
{
  int (*termlist)[maxsym + 2];
  double (*termvalue);
  int (*poly)[maxdegree];
  int degterms[maxdegree];
  int maxterms;
  int maxpoly;
  int toterms;
  int opamps;
}
vardef;


vardef vars[NVARS];

int topological_fun[6];

int term[maxsym], cancelfactor[maxsym], cancelcoeff, cancelsign, cancelexp;
int branchmap[maxsym], dbranchmap[maxsym];

int it, nt, vstk[stk1][nmax], estk[stk1][emax], state, mtree[nmax];
#ifdef COMBINED_TREES
int raw_nt, common_nt, usable_nt;
#endif
int mn, me, mr, mq;

clock_t start_time;


char* topo_names[] = 
{
  "V",
  "Wijkl",
  "Wijij",
  "Wklkl",
  "Wklij",
  "Uijkl",
}
;

typedef  struct f_tab
{
  int fn_no;
  char* fn_name;
  int fn_p1p;
  int fn_p1m;
  int fn_p2p;
  int fn_p2m;
  char* var1_name;
  int var1_p1p;
  int var1_p1m;
  char* var2_name;
  int  var2_p1p;
  int  var2_p1m;
  char* var3_name;
  int  var3_p1p;
  int  var3_p1m;
  int sign;
  int num_form;
  int denom_form;
}
f_tab;

/*control table for driving analysis:  its order is the same as the par_names list;
  fn_no is a vestige from the function ordering of earlier versions */

f_tab function_table[] = 
{
  {
    20,"g",0,1,0,1,"i",0,1,"v",0,1,"i",2,3, 1,V,wijij
  }
  ,
  {
    3 ,"g",0,1,2,3,"i",0,1,"i",2,3,"v",0,1,-1,wklij,wijij
  }
  ,
  {
    0 ,"g",2,3,0,1,"v",2,3,"v",0,1,"i",2,3, 1,wijkl,wijij
  }
  ,
  {
    21,"g",2,3,2,3,"v",2,3,"i",2,3,"v",0,1, 1,uijkl,wijij
  }
  ,
  {
    22,"h",0,1,0,1,"v",0,1,"i",0,1,"v",2,3, 1,uijkl,wklkl
  }
  ,
  {
    1 ,"h",0,1,2,3,"v",0,1,"v",2,3,"i",0,1, 1,wklij,wklkl
  }
  ,
  {
    2 ,"h",2,3,0,1,"i",2,3,"i",0,1,"v",2,3,-1,wijkl,wklkl
  }
  ,
  {
    23,"h",2,3,2,3,"i",2,3,"v",2,3,"i",0,1, 1,V,wklkl
  }
  ,
  {
    4 ,"y",0,1,0,1,"i",0,1,"v",0,1,"v",2,3, 1,wklkl,uijkl
  }
  ,
  {
    6 ,"y",0,1,2,3,"i",0,1,"v",2,3,"v",0,1,-1,wklij,uijkl
  }
  ,
  {
    5 ,"y",2,3,0,1,"i",2,3,"v",0,1,"v",2,3,-1,wijkl,uijkl
  }
  ,
  {
    7 ,"y",2,3,2,3,"i",2,3,"v",2,3,"v",0,1, 1,wijij,uijkl
  }
  ,
  {
    8 ,"z",0,1,0,1,"v",0,1,"i",0,1,"i",2,3, 1,wijij,V
  }
  ,
  {
    10,"z",0,1,2,3,"v",0,1,"i",2,3,"i",0,1, 1,wklij,V
  }
  ,
  {
    9 ,"z",2,3,0,1,"v",2,3,"i",0,1,"i",2,3, 1,wijkl,V
  }
  ,
  {
    11,"z",2,3,2,3,"v",2,3,"i",2,3,"i",0,1, 1,wklkl,V
  }
  ,
  {
    24,"A",2,3,0,1,"v",0,1,"v",2,3,"i",2,3, 1,wijij,wijkl
  }
  ,
  {
    25,"B",2,3,0,1,"v",0,1,"-i",2,3,"v",2,3, 1,uijkl,wijkl
  }
  ,
  {
    26,"C",2,3,0,1,"i",0,1,"v",2,3,"i",2,3, 1,V,wijkl
  }
  ,
  {
    27,"D",2,3,0,1,"i",0,1,"-i",2,3,"v",2,3, 1,wklkl,wijkl
  }
  ,
  {
    28,"a",0,1,2,3,"v",2,3,"v",0,1,"i",0,1, 1,wklkl,wklij
  }
  ,
  {
    29,"b",0,1,2,3,"v",2,3,"-i",0,1,"v",0,1, 1,uijkl,wklij
  }
  ,
  {
    30,"c",0,1,2,3,"i",2,3,"v",0,1,"i",0,1, 1,V,wklij
  }
  ,
  {
    31,"d",0,1,2,3,"i",2,3,"-i",0,1,"v",0,1, 1,wijij,wklij
  }
  ,
/* the following functions use an embedding network for analysis*/
  {
    12,"s",0,1,0,1," ",0,0," ",0,0," ",0,0,-1,wklkl,saved_uijkl
  }
  ,
  {
    13,"s",0,1,2,3," ",0,0," ",0,0," ",0,0, 1,wklij,saved_uijkl
  }
  ,
  {
    14,"s",2,3,0,1," ",0,0," ",0,0," ",0,0, 1,wijkl,saved_uijkl
  }
  ,
  {
    15,"s",2,3,2,3," ",0,0," ",0,0," ",0,0,-1,wijij,saved_uijkl
  }
  ,
  {
    16,"F",0,1,0,1," ",0,0," ",0,0," ",0,0,-1,V,saved_wijkl
  }
  ,
  {
    17,"F",0,1,2,3," ",0,0," ",0,0," ",0,0,-1,wklkl,saved_wijkl
  }
  ,
  {
    18,"F",2,3,0,1," ",0,0," ",0,0," ",0,0, 1,wijij,saved_wijkl
  }
  ,
  {
    19,"F",2,3,2,3," ",0,0," ",0,0," ",0,0, 1,uijkl,saved_wijkl
  }
  ,
}
;

char* par_names[2][(sizeof(function_table)/sizeof(f_tab))] = 
{
  {
    "g11","g12","g21","g22","h11","h12","h21","h22",
    "y11","y12","y21","y22","z11","z12","z21","z22",
    "A","B","C","D","a","b","c","d",
    "s11","s12","s21","s22","f11","f12","f21","f22"
  }
  ,
  {
    "G11","G12","G21","G22","H11","H12","H21","H22",
    "Y11","Y12","Y21","Y22","Z11","Z12","Z21","Z22",
    "A21","B21","C21","D21","a12","b12","c12","d12",
    "S11","S12","S21","S22","F11","F12","F21","F22" 
  }
}
;

int par_list[(sizeof(function_table)/sizeof(f_tab))];

#ifdef COMBINED_TREES
/* this maps the F1 and F2 edge combinations present in products to
   the topological formulas for the two categories of functions */
int topomap[8] ={V, wijij, wklkl, uijkl, -1, wijkl, wklij, -1};
#endif

#define VARLIST(x) & vars[x].termlist,& vars[x].termvalue, & vars[x].poly, vars[x].degterms,& vars[x].toterms,& vars[x].maxterms,& vars[x].maxpoly

#define NUM_VARS VARLIST(numerator)
#define DENOM_VARS VARLIST(denominator)


#define PARAMETERLIST int (**ptermlist)[maxsym + 2], double (**ptermvalue), int (**ppoly)[maxdegree], int *degterms, int *toterms, int *maxtermlist, int *maxpoly



/* input functions */
int main(void);
int parse_input(void);
int getstr(int *len, int lim, char *str);
int getint(int *val);
int getdouble(double *val);
void help(void);
int nodemap (int node);
int internal_node (void);
void eval_maps(void);
int getsym (int i);

/* analysis functions */
void analyze_network(void);
#ifdef COMBINED_TREES
void evaluate_fn (int e, int a, int r, int toposet);
#else
void evaluate_fn (int e, int a, int r, int topofun);
#endif
#define ev_fn(n) evaluate_fn(e,a,r, n)
void embed_network_for_s_parameters(void);
void init (PARAMETERLIST);
void release_topo_data(void);
void release (PARAMETERLIST);
int product (int minty_nodes, int edges, int actual_nodes, int form, int br, int topofun);
void minty (void);
void treecheck (int r, int *d1, int *d2, int *tree, int *perm);

/* admittance product manipulation functions */
int getexpr (int sign, int deg, int factors, int *factorlist, int edges, PARAMETERLIST);
int resize(void ** ptr, int * count, int module);

int print_treebuf (int sign, int num_fun, int denom_fun);
int copy(int sign, int dest, int source);
void combineinit (void);
int fixup (void);
void printexp (PARAMETERLIST);
void eval_branchtag (PARAMETERLIST);
int correctunits (PARAMETERLIST);
void makecancellationlist (PARAMETERLIST);

int 
main ()
{

  start_time = clock();

  printf ("{program file name tree45a.c}\n");
  inptno = 0;

  for(;0 == parse_input();)
    analyze_network();

  printf("{elapsed time: %d/%d sec.}\n",(clock() - start_time),CLOCKS_PER_SEC);

  return(0);
}

/* parse_input uses getstr and getint instead of scanf in order to allow
   explicit control of comments. */
/* the input netlist generates edge lists with ( 2- or 4-) node entries,
classified by passive 2 node edges, and active 4 node double edges. */

int parse_input(void)
{
  int i, j, k, scan, node1, node2, node3, node4, int1, int2, ch0, ch1,
    f, par_flag, value_index = 1;
  char string[128];

  printf ("\ninput data:\n");

  inptno++;

  test = 0;
  normalization = 0;
  normalnet1 = 0;
  normalnet2 = 0;
  normalnet3 = 0;
  normalnet4 = 0;
  opampcnt = 0;
  p = 1;			/* first edge is always dummy */
  branches = 1;
  n = 0;
  int_node = 999;
  a = 0;
  f = 0;


  for (i = 0; i < nmax; i++)
    nodelist[i] = 0;

  for(i = 0; i < (sizeof(function_table)/sizeof(f_tab)); i++)
    par_list[i] = 1;

  for(i = 0; i < emax; i++)
    value[i] = 0.0;

  par_flag = 0;

  for (scan = 1; scan == 1;) 
  {

    if (getstr( &i, sizeof(string), &string[0]) == EOF)
      return (1);
    if (i > BRLEN )
    {
      printf("following branch name was truncated to %d characters:\n",BRLEN );
      i = BRLEN;
    }

    string[i] = 0;
    for(;i >= 0;i--)branch[branches + 1][i] = string[i];


    printf ("%s", &branch[branches + 1][0]);

    ch0 = branch[branches + 1][0];
    ch1 = branch[branches + 1][1];

    if ((ch0 != 'v') && (ch0 != 'V') ) value_index = branches + 1;

    if ((((ch0 == 'g') || (ch0 == 'G') || (ch0 == 'r') || (ch0 == 'R')) && ((ch1 == 'm') || (ch1 == 'M'))) ||
      (ch0 == 'o') || (ch0 == 'O') || (ch0 == 'u') || (ch0 == 'U') || (ch0 == 'b') || (ch0 == 'B')) 
    {

      if (getint(&node1) == EOF)return(1);
      printf("%u",node1);
      if (getint(&node2) == EOF)return(1);
      printf("%u",node2);
      if (getint(&node3) == EOF)return(1);
      printf("%u",node3);
      if (getint(&node4) == EOF)return(1);
      printf("%u",node4);

      switch (ch0) 
      {
      case 'o':
      case 'O':
        opampcnt++;
      case 'g':
      case 'G':
        branches++;
        a++;
        a0[a] = branches;
        a1[a] = nodemap (node1);
        a2[a] = nodemap (node2);
        a3[a] = nodemap (node3);
        a4[a] = nodemap (node4);
        break;
      case 'r':
      case 'R':

        branches++;
        p++;
        p0[p] = branches;
        p1[p] = nodemap (node3);
        p2[p] = internal_node ();

        branches++;
        a++;
        a0[a] = branches;
        a1[a] = internal_node ();
        a2[a] = nodemap (node2);
        a3[a] = nodemap (node4);
        a4[a] = p2[p];
        branch[branches][0] = 'a';
        branch[branches][1] = 0;

        opampcnt++;
        branches++;
        a++;
        a0[a] = branches;
        a1[a] = nodemap (node1);
        a2[a] = nodemap (node2);
        a3[a] = nodemap (node2);
        a4[a] = a1[a - 1];
        branch[branches][0] = 'o';
        branch[branches][1] = 0;


        opampcnt++;
        branches++;
        a++;
        a0[a] = branches;
        a1[a] = p2[p];
        a2[a] = nodemap (node4);
        a3[a] = nodemap (node4);
        a4[a] = nodemap (node3);
        branch[branches][0] = 'o';
        branch[branches][1] = 0;

        branches++;
        p++;
        p0[p] = branches;
        p1[p] = nodemap (node1);
        p2[p] = a1[a - 2];
        branch[branches][0] = 'a';
        branch[branches][1] = 0;

        break;

      case 'u':
      case 'U':
        branches++;
        a++;
        a0[a] = branches;
        a1[a] = internal_node ();
        a2[a] = nodemap (node2);
        a3[a] = nodemap (node3);
        a4[a] = nodemap (node4);

        opampcnt++;
        branches++;
        a++;
        a0[a] = branches;
        a1[a] = nodemap (node1);
        a2[a] = nodemap (node2);
        a3[a] = nodemap (node2);
        a4[a] = a1[a - 1];
        branch[branches][0] = 'o';
        branch[branches][1] = 0;

        branches++;
        p++;
        p0[p] = branches;
        p1[p] = a1[a - 1];
        p2[p] = nodemap (node1);
        branch[branches][0] = 'a';
        branch[branches][1] = 0;

        break;

      case 'b':
      case 'B':
        branches++;
        a++;
        a0[a] = branches;
        a1[a] = nodemap (node2);
        a2[a] = nodemap (node1);
        a3[a] = internal_node ();
        a4[a] = nodemap (node4);

        opampcnt++;
        branches++;
        a++;
        a0[a] = branches;
        a1[a] = a3[a - 1];
        a2[a] = nodemap (node4);
        a3[a] = nodemap (node4);
        a4[a] = nodemap (node3);
        branch[branches][0] = 'o';
        branch[branches][1] = 0;

        branches++;
        p++;
        p0[p] = branches;
        p1[p] = nodemap (node3);
        p2[p] = a3[a - 1];
        branch[branches][0] = 'a';
        branch[branches][1] = 0;

        break;

      }
    }
    else if ((ch0 == 'm') || (ch0 == 'M')) 
    {

      if (getint(&node1) == EOF)return(1);
      printf("%u",node1);
      if (getint(&node2) == EOF)return(1);
      printf("%u",node2);
      if (getint(&node3) == EOF)return(1);
      printf("%u",node3);
      if (getint(&node4) == EOF)return(1);
      printf("%u",node4);


      int1 = internal_node ();
      int2 = internal_node ();


      branches++;
      p++;
      p0[p] = branches;
      p1[p] = nodemap (node3);
      p2[p] = internal_node ();
      k = branches;

      branches++;
      a++;
      a0[a] = branches;
      a1[a] = internal_node ();
      a2[a] = int2;
      a3[a] = int1;
      a4[a] = p2[p];
      branch[branches][0] = 'a';
      branch[branches][1] = 0;

      opampcnt++;
      branches++;
      a++;
      a0[a] = branches;
      a1[a] = nodemap (node1);
      a2[a] = int2;
      a3[a] = int2;
      a4[a] = a1[a - 1];
      branch[branches][0] = 'o';
      branch[branches][1] = 0;


      opampcnt++;
      branches++;
      a++;
      a0[a] = branches;
      a1[a] = p2[p];
      a2[a] = int1;
      a3[a] = int1;
      a4[a] = nodemap (node3);
      branch[branches][0] = 'o';
      branch[branches][1] = 0;

      branches++;
      p++;
      p0[p] = branches;
      p1[p] = nodemap (node1);
      p2[p] = a1[a - 2];
      branch[branches][0] = 'a';
      branch[branches][1] = 0;


/* second part */

      branches++;
      p++;
      for (i = 0; (branch[branches][i] = branch[k][i]) != 0; i++)
        ;
      p0[p] = branches;
      p1[p] = int2;
      p2[p] = internal_node ();

      branches++;
      a++;
      a0[a] = branches;
      a1[a] = internal_node ();
      a2[a] = nodemap (node4);
      a3[a] = nodemap (node2);
      a4[a] = p2[p];
      branch[branches][0] = 'a';
      branch[branches][1] = 0;

      opampcnt++;
      branches++;
      a++;
      a0[a] = branches;
      a1[a] = int1;
      a2[a] = nodemap (node4);
      a3[a] = nodemap (node4);
      a4[a] = a1[a - 1];
      branch[branches][0] = 'o';
      branch[branches][1] = 0;


      opampcnt++;
      branches++;
      a++;
      a0[a] = branches;
      a1[a] = p2[p];
      a2[a] = nodemap (node2);
      a3[a] = nodemap (node2);
      a4[a] = int2;
      branch[branches][0] = 'o';
      branch[branches][1] = 0;

      branches++;
      p++;
      p0[p] = branches;
      p1[p] = int1;
      p2[p] = a1[a - 2];
      branch[branches][0] = 'a';
      branch[branches][1] = 0;



    }
    else if (((ch0 == 'g') || (ch0 == 'G')) && ((ch1 == 'y') || (ch1 == 'Y'))) 
    {

      if (getint(&node1) == EOF)return(1);
      printf("%u",node1);
      if (getint(&node2) == EOF)return(1);
      printf("%u",node2);
      if (getint(&node3) == EOF)return(1);
      printf("%u",node3);
      if (getint(&node4) == EOF)return(1);
      printf("%u",node4);


      branches++;
      a++;
      a0[a] = branches;
      a1[a] = nodemap (node1);
      a2[a] = nodemap (node2);
      a3[a] = nodemap (node3);
      a4[a] = nodemap (node4);

      branches++;
      for (i = 0; (branch[branches][i] = branch[branches - 1][i]) != 0; i++)
        ;

      a++;
      a0[a] = branches;
      a1[a] = nodemap (node4);
      a2[a] = nodemap (node3);
      a3[a] = nodemap (node1);
      a4[a] = nodemap (node2);



    }
    else if ((ch0 == 'f') || (ch0 == 'F')) 
    {
      if (f == 1) 
      {
        printf ("error-- more than one port indicator is used\n");
        return (1);
      }
      ;
      f = 1;

      if (getint(&ni1) == EOF)return(1);
      printf("%u",ni1);
      if (getint(&nj1) == EOF)return(1);
      printf("%u",nj1);
      if (getint(&nk1) == EOF)return(1);
      printf("%u",nk1);
      if (getint(&nl1) == EOF)return(1);
      printf("%u",nl1);

      ni = nodemap (ni1);
      nj = nodemap (nj1);
      nk = nodemap (nk1);
      nl = nodemap (nl1);
    }
    else if ((ch0 == 'p') || (ch0 == 'P')) 
    {
      if (par_flag == 0) 
      {
        par_flag = 1;
        for(i = 0; i < (sizeof(function_table)/sizeof(f_tab)); i++)
          par_list[i] = 0;
      }
      ;

    if (getstr( &i, sizeof(string), &string[0]) == EOF)
      return (1);
      printf ("%s", string);


      i = 0;
      while (i < (sizeof(function_table)/sizeof(f_tab))) 
      {
        j = 0;
        while ((string[j] == par_names[0][i][j]) && (par_names[0][i][j] != 0))
          j++;
        if (string[j] == par_names[0][i][j]) 
        {
          par_list[i] = 1;
          break;
        }
        ;
        i++;
      }
      ;
      if(i == (sizeof(function_table)/sizeof(f_tab)))
      {

        i = 0;
        while (i < (sizeof(function_table)/sizeof(f_tab))) 
        {
          j = 0;
          while ((string[j] == par_names[1][i][j]) && (par_names[1][i][j] != 0))
            j++;
          if (string[j] == par_names[1][i][j]) 
          {
            par_list[i] = 1;
            break;
          }
          ;
          i++;
        }
        ;
        if(i == (sizeof(function_table)/sizeof(f_tab)))
          printf(" %s does not match the list of parameter names, see help info.\n",string);


      }
      ;


    }
    else if ((ch0 == 'g') || (ch0 == 'G') ||
      (ch0 == 'z') || (ch0 == 'Z') ||
      (ch0 == 'y') || (ch0 == 'Y') ||
      (ch0 == 'r') || (ch0 == 'R') ||
      (ch0 == 'l') || (ch0 == 'L') ||
      (ch0 == 'c') || (ch0 == 'C')) 
    {

      if (getint(&node1) == EOF)return(1);
      printf("%u",node1);
      if (getint(&node2) == EOF)return(1);
      printf("%u",node2);


      branches++;
      p++;
      p0[p] = branches;
      p1[p] = nodemap (node1);
      p2[p] = nodemap (node2);
    }
    else if ((ch0 == 'n') || (ch0 == 'N')) 
    {

     if(ch1 == 0)
      {
      if (getint(&normalization) == EOF)return(1);
      printf("%u",normalization);
      } else {
        if ((ch1 == '1') ||  (ch1 == '2')) {
           if (getint(&node1) == EOF)return(1);
           printf("%u",node1);
           if (getint(&node2) == EOF)return(1);
           printf("%u",node2);

           if (ch1 == '1') {
             normalnet1  = nodemap (node1);
             normalnet2  = nodemap (node2);
           } else  {
             normalnet3  = nodemap (node1);
             normalnet4  = nodemap (node2);
           }


        } else {
           printf("n1 node# node#  or n2 node# node# is proper usage for\n");
           printf("scattering parameter normalizing network references\n");
           help();
           return(1);
       }         
      }
    }
    else if ((ch0 == 'e') || (ch0 == 'E')) 
    {
/* dummy edge (handle) and extra node-- see comments for function minty */
      p0[1] = 1;
      p1[1] = n + 1;
      p2[1] = 1;
      branch[1][0] = 'x';
      branch[1][1] = 0;
      n++;
      scan = 0;
    }
    else if ((ch0 == 'h') || (ch0 == 'H')) 
    {
      help();
      return (1);
    }
    else if ((ch0 == 't') || (ch0 == 'T')) 
    {
      if (getint(&test) == EOF)return(1);
      printf("%u",test);
    }
    else if ((ch0 == 'v') || (ch0 == 'V')) 
    {
      if (getdouble(&value[value_index]) == EOF)return(1);
      printf("%g",value[value_index]);
    }
    else
    {
      printf ("unrecognized element prefix-- '%s': must be r,l,c,g,y,z,gm,gy,o,u,b,rm,m,n,f,e,v\n", &branch[branches + 1][0]);
      printf ("h as input prints help\n");
      return (1);
    }
    ;



    if (p >= (pmax - s_passive)) 
    {
      printf ("number of one-port elements(= %d) exceeds limit of %d\n or improper input applied\n", p, pmax - (1 + s_passive));
      return (1);
    }
    ;

    if (a >= (actmax - s_active)) 
    {
      printf ("number of two-port elements(= %d) exceeds limit of %d\n or improper input applied\n", a, actmax - (1 + s_active));
      return (1);
    }
    ;

    if (n >= (nmax - s_nodes)) 
    {
      printf ("number of nodes(= %d) exceeds limit of %d\n or improper input applied\n", a, nmax - (1 + s_nodes));
      return (1);
    }
    ;


  }

  /* copy literal values to all instances of same named branches */

     for(k=0;k<=branches;k++){
      if (value[k] != 0){
        for (i=0;i<=branches;i++){
	  if (i == k) continue;
          for (j=0;(branch[i][j] != 0) && (branch[k][j] != 0);j++){
           if (branch[i][j] != branch[k][j]) break;
          };
         if (branch[i][j] == branch[k][j]) value[i] = value[k];
       };
     };
    };


  printf ("\n\n");

  if (f == 0) 
  {
    printf ("please provide a port node indication: f (lower case or upper case),\n");
    printf ("whitespace, +port1 node#, whitespace, -port1 node#, whitespace, +port2 node#, whitespace, -port2 node#\n\r");
    printf ("example:\n");
    printf ("f 1 0 2 4\n");

    return (1);
  }
  ;
  if ((test & PRINT_STATS) != 0) 
  {
    printf (" currently configured resource limits:\n");
    printf ("branch name list limit:%d, used:%d\n", emax - (s_passive + s_active), branches);
    printf ("node list limit:%d, used:%d\n", nmax - s_nodes, n);
    printf ("passive edges limit:%d, used:%d\n", pmax - s_passive, p);
    printf ("active edges limit:%d, used:%d\n", actmax - s_active, a);
  }
  ;

  e = p;
  r = n - 1;

  eval_maps();



  if ((test & INPUT_DUMP) != 0) 
  {
    printf ("Internal representation of data including generated elements and reassigned node#s:\n");
    printf ("edge,name,node#'s...\n");
    for (i = 1, j = 1; i <= p; i++, j++) 
    {
      printf ("%3d %s %d %d\n", j, &branch[p0[i]][0], p1[i], p2[i]);
    }
    ;
    for (i = 1; i <= a; i++, j++) 
    {
      printf ("%3d %s %d %d %d %d\n", j, &branch[a0[i]][0], a1[i], a2[i], a3[i], a4[i]);
    }
    ;
    printf ("%3d f %d %d %d %d\n", j, ni, nj, nk, nl);
    printf ("internal node#  input node#:\n");
    for (i = 1; i <= n; i++)
      printf ("         %5d    %d\n", i, nodelist[i]);
    printf ("\nedge namelist: index, value, numeric value\n");
    for (i = 1; i <= branches; i++)
      if ( value[i] != 0) {printf ("%3d, %s %g\n", i, &branch[i][0], value[i]);}
      else {printf ("%3d, %s\n", i, &branch[i][0]);};
    printf ("\n");
  }
  ;

  return(0);
}

void help()
{
  int i;
  int port_node[4] = 
    {
    1,1,2,2
  }
  ;
  char * help1[]={

      "\n\nhelp:\n",
      "usage:\ntree45a <inputfile >outputfile\n",
      "\nsymbolic linear network analysis program written by Jeff Furman, AD6MX, 5-4-2001 vers 45a\n",
      "Mayeda's method for active networks is implemented using a\n",
      "modified Chan's implementation of Minty's algorithm for evaluating\n",
      "spanning trees of network graphs. Lin, pp. 115-117 has Chan's implementation.\n",
      "\nnetlist element definitions, hints, references, and examples follow:\n",
      "\nthe netlist constitutes entries of one- and two- port elements,\n",
      "a port designator entry, optional two-port parameter entries,\n",
      "an optional scattering parameter normalizing entry, an optional test/output\n",
      "bitmap entry, and all this followed by an end terminating entry.\n",
      "the order of these entries is not critical, except for the end entry.\n",
      "multiple netlists are separated by end; each netlist analysis is independent\n",
      "bracketed comments may be inline with other entries, they are not considered to be\n",
      "whitespace.\n",
      "\ncomments are bracketed; and may be nested: {comments... {comments... } ...}\n",
      "\ninput lines may be concatenated when separated by whitespace:\n",
      "space,tab,cr,lf,comma,semicolon\n",
      "\nnode numbers are arbitrary in the range 0 to 999\n",
      "\nThis version (44a and later) has a feature allowing numeric values to be assigned.\n",
      "This allows combination of terms when otherwise distinct symbolic references have\n",
      "numeric values. Any passive or active element may be assigned a floating point numeric\n",
      "value. The symbolic references are replaced by the numeric values in the output;\n",
      "the first instance of a numeric value for a particular element symbolic reference\n",
      "is assigned to all the instances of that symbolic reference. Numeric values, denoted\n",  
      "by v or V, follow the corresponding element declaration, and are separated by whitespace.\n",
      "<one- or two- port element declaration> <whitespace> v <whitespace> <floating point number> <whitespace>\n",
      "\na few examples:\n",
      "\nr23 8 4 v 1e3\n",
      "r3 6 4 V -11\n",
      "L 6 2 v 100\n",
      "GM5 1 3 2 4 v 38.4\n",
      "C 6 3 V 4.7e-12\n", 
      "\none-port (two-terminal) passive elements:\n",
      "r,l,c,g,y,z (lower case or upper case) prefix, null up to 8 character suffix,\n",
      "whitespace, first node#,whitespace,second node#\n",
      "r<suffix> <n1#> <n2#>\n",
      "l<suffix> <n1#> <n2#>\n",
      "c<suffix> <n1#> <n2#>\n",
      "g<suffix> <n1#> <n2#>\n",
      "y<suffix> <n1#> <n2#>\n",
      "z<suffix> <n1#> <n2#>\n",
      "\nexamples--\n",
      "r 0 4\n",
      "L23a 7 41\n",
      "g2 3 8\n",
      "(gm, gy, and rm are reserved for two-port elements)\n",
      "\ntwo-port (four-terminal) active elements:\n",
      "gm,gy,o,u,b,rm,m (lower case or upper case) prefix, null up to 8 character suffix,\n",
      "whitespace, +output node#, whitespace, -output node#, whitespace, +input node#, whitespace, -input node#,\n",
      "gm<suffix> <+out#> <-out#> <+in#> <-in#>\n",
      "gy<suffix> <+out#> <-out#> <+in#> <-in#>\n",
      "o<suffix> <+out#> <-out#> <+in#> <-in#>\n",
      "u<suffix> <+out#> <-out#> <+in#> <-in#>\n",
      "b<suffix> <+out#> <-out#> <+in#> <-in#>\n",
      "rm<suffix> <+out#> <-out#> <+in#> <-in#>\n",
      "m<suffix> <+out#> <-out#> <+in#> <-in#>\n",
      "positive current enters +nodes except port 2 in ABCD parameters and port 1 in abcd parameters.\n",
      "gm denotes voltage controlled current source,\n",
      "o denotes ideal op amp,\n",
      "u (mu) denotes voltage controlled voltage source,\n",
      "b (beta) denotes current controlled current source,\n",
      "rm denotes current controlled voltage source,\n",
      "gy denotes a gyrator, and\n",
      "m denotes a mutual inductance\n",
      "\ncontrolled source orientation:\n",
      "      _             _             _             _       \n",
      "     / \\           / \\           / \\           / \\      \n",
      "(+)_/+ -\\_(-);(+)_/+ -\\_(-);(+)_/-->\\_(-);(+)_/-->\\_(-);\n",
      "    \\rm /         \\ u /         \\gm /         \\ b /     \n",
      "     \\_/           \\_/           \\_/           \\_/      \n",
      "examples--\n",
      "gm2 1 2 3 4\n",
      "U3 3 0 7 5\n",
      "o 6 3 2 9\n",
      "O 3 1 3 4\n",
      "\nmandatory port designator-- \n",
      "f (lower case or upper case),\n",
      "whitespace, +port1 node#, whitespace, -port1 node#, whitespace, +port2 node#, whitespace, -port2 node#\n",
      "f <+p1#> <-p1#> <+p2#> <-p2#>\n",
      "for ports expressed as f <i> <j> <k> <l>\n",
      "where <i> <j> <k> <l> are numeric node numbers labeling the port terminals,\n",
      "then port 1 is denoted by (i,j) and port 2 is denoted by (k,l)\n",
      "examples: y(i,j;k,l) == y12\n",
      "g(k,l;i,j) == g21 etc.\n",
      "concrete example: for\n",
      "f 3 7 9 10\n",
      "p g21\n",
      "this would be:\n",
      "g(9, 10; 3, 7) = v(9, 10)/v(3, 7) for i(9, 10) = 0\n",
      "example--\n",
      "f 1 3 2 4\n",
      "\ntwo-port parameter notation:\n",
      "\noptional parameter designator-- may be repeated for multiple parameters;\n",
      "if it is omitted, default is evaluating all 32 parameter functions:\n",
      "p (lower or upper case), whitespace, one of these parameter names:\n",
      "g11 g12 g21 g22 h11 h12 h21 h22 y11 y12 y21 y22 z11 z12 z21 z22\n",
      "A B C D a b c d s11 s12 s21 s22 f11 f12 f21 f22\n",
      "case of letters is sensitive for ABCD and abcd only\n",
      "\ny denotes short circuit admittance parameters,\n",
      "z denotes open  circuit impedance  parameters,\n",
      "h denotes hybrid parameters,\n",
      "g denotes inverse hybrid parameters,\n",
      "A,B,C,D denotes forward chain or transmission parameters,\n",
      "a,b,c,d denotes reverse chain parameters,\n",
      "s denotes scattering parameters,\n",
      "F denotes scattering forward chain parameters,\n",
      "examples--\np g21\n",
      "P y11\n",
      "p Y12\n",
      "p a p b p c p d p g21\n",
      "p A\n",
      "\nn # or N # specifies simple normalization for s and F parameters: 0- Y0, 1- Z0, 2(or greater)- unity\n",
      "if n 0 is omitted, the default is Y0\n",
      "example--\nn 1\n",
      "\nn1 node# node# or N1 node# node# for port 1 and n2 node# node# or N2 node# node# for port 2\n"
      "specifies a normalization network for each port for s and F parameters; if no network nodes are\n",
      "defined for a port, the normalization for that port is as above.\n"
      "these normalization networks seem to be floating, but they connect to the augmentation circuitry.\n",
      "version 45a currently permits only non-reactive (resistive only) normalization networks.\n",
      "general networks require transforming immittances to their conjugates.\n",
      "example--\nN1 7 18\nn2 11 5\nr01 7 18 v 50\nr02 11 5 v 75\n",
      "\nt # before end line shows intermediate results: 1-input data dump,2-raw tree listing,\n8-matched trees and permutation listing,16-mixed unit trees,32-admittance trees,64-resource usage\n",
      "example--\n",
      "t 65\n",
      "\ne or end  terminates input\n",
      "example--\n",
      "end\n",
      "\nhints:\n",
      "\nfloating or isolated network regions must be avoided-- they\n",
      "exist when only two-port elements connect them to the remainder of the\n",
      "network. a remedy is sharing a common node (preferred, since it reduces the\n",
      "graph complexity), or connecting these regions with single branches\n",
      "these single branches eventually cancel, so they don't appear in the results,\n",
      "but, they take their toll in increasing computation time. a rule of thumb is\n",
      "all nodes must be connected in a single graph, when two-port\n",
      "elements are considered to be two separate one-port elements.\n",
      "ideal opamps have the properties of nullors-- the polarity of their inputs\n",
      "and outputs doesn't affect network expressions. stability of actual opamp\n",
      "networks must be established by alternate models. see Lin, section 8.3.1, pg.234\n",
      "example networks at the end demonstrate this\n",
      "\nresource burden of input element types:\n",
      "elem prefix, branch name list, node list, passive edges, active edges\n",
      "r,l,c,g,y,z        1           no extra         1             0\n",
      " GM, O             1           no extra         0             1\n",
      " GY                2           no extra         0             2\n",
      "  U, B             3            1 extra         1             2\n",
      " RM                5            2 extra         2             3\n",
      "  M               10            6 extra         4             6\n",
      "any s or F param  12            8 extra         8             4\n",
      "each normalizing   1           no extra         0             1\n",
      "  network\n",
      " the extra nodes and branches are invisible in the output\n",
      " currently configured resource limits exclusive of requirements\n",
      "for s and F parameters:\n",
      };

 char * help2[]={
      "\n",
      "\nparameter summary:\n",
      "these parameters are directly computed:\n",
      "g11, g12, g21, g22, h11, h12, h21, h22,\n",
      "y11, y12, y21, y22, z11, z12, z21, z22,\n",
      "A, B, C, D, a, b, c, d,\n",
      "s11, s12, s21, s22, F11, F12, F21, F22.\n",
      };

 char * help3[]={
      "\n",
      "references:\n",
      "Huelsman, L. P.; \"Circuits, Matrices, and Linear Vector Spaces\"; pp. 57-63\n",
      "Weinberg, L.; Chapter 4 in Shea, R. F., ed.; \"Amplifier Handbook\" pp. 4-27 - 4-31.\n",
      "Lin, Pen-Min; \"Symbolic Network Analysis\"; pp. 115-117, pp. 213-219, pg. 234\n",
      "Mayeda, W.; \"Graph Theory\"; ch. 7 and 8; pp. 252-341\n",
      "Percival, W. S.;\n",
      "   \"The Solution of Passive Electrical Networks by Means of Mathematical Trees\";\n",
      "   Journal of IEE V. 100 part 3 (May, 1953) pp.143-150\n",
      "\n",
      "\n",
      "sample netlists-- copy and paste to a file to use as input to see output\n\n",
      "{Sallen-Key active filters.\n",
      "see IRE Transactions on Circuit Theory,\n",
      "CT-2, pp 74-85 (March 1955).\n",
      "These are numbered according to the charts\n",
      "in the paper. Most of these use a non-\n",
      "inverting controlled source, which is easy\n",
      "to implement. A few, numbers 5, 7, 12, and\n",
      "15 require an inverting high input impedance\n",
      "amplifier, which generally requires more\n",
      "circuitry.}\n\n",

      "{circuit 1.\n",
"Vo/Vi = k/(s^2 + d*s +1)\n",
"lowpass}\n",
"r1 1 4\nr2 4 5\nc1 2 4\nc2 5 3\nU 2 3 5 3\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 3.\n",
"Vo/Vi = k*s^2/(s^2 + d*s +1)\n",
"highpass}\n",
"c1 1 4\nc2 4 5\nr1 4 2\nr2 5 3\nU 2 3 5 3\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 5.\n",
"Vo/Vi = k*s/(s^2 + d*s +1)\n",
"bandpass}\n",
"r1 1 4\nr2 5 3\nc1 4 2\nc2 4 5\nU 2 3 3 5\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 7.\n",
"Vo/Vi = k*s/(s^2 + d*s +1)\n",
"bandpass}\n",
"c1 1 4\nc2 5 3\nr1 4 2\nr2 4 5\nU 2 3 3 5\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 10.\n",
"Vo/Vi = k*(s + b)/(s^2 + d*s +1)\n",
"asymmetric bandpass}\n",
"r1 2 4\nr2 1 5\nc1 4 3\nc2 5 4\nU 2 3 5 3\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 11.\n",
"Vo/Vi = k*(s + b)/(s^2 + d*s +1)\n",
"asymmetric bandpass}\n",
"r1 1 5\nr2 2 4\nc1 5 3\nc2 4 5\nU 2 3 5 3\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 12.\n",
"Vo/Vi = k*(s + b)/(s^2 + d*s +1)\n",
"asymmetric bandpass}\n",
"r1 1 4\nr2 1 5\nc1 2 4\nc2 4 5\nU 2 3 3 5\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 13.\n",
"Vo/Vi = k*s*(s + b)/(s^2 + d*s +1)\n",
"asymmetric bandpass}\n",
"r1 3 4\nr2 4 5\nc1 2 4\nc2 1 5\nU 2 3 5 3\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 14.\n",
"Vo/Vi = k*s*(s + b)/(s^2 + d*s +1)\n",
"asymmetric bandpass}\n",
"r1 5 3\nr2 2 4\nc1 1 5\nc2 4 5\nU 2 3 5 3\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 15.\n",
"Vo/Vi = k*s*(s + b)/(s^2 + d*s +1)\n",
"asymmetric bandpass}\n",
"r1 2 4\nr2 4 5\nc1 1 4\nc2 1 5\nU 2 3 3 5\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 16.\n",
"Vo/Vi = k*(s^2 + b*s +1)/(s^2 + d*s +1)\n",
"biquad}\n",
"r1 2 4\nr2 1 5\nc1 1 4\nc2 4 5\nU 2 3 5 3\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 17.\n",
"Vo/Vi = k*(s^2 + b*s +1)/(s^2 + d*s +1)\n",
"biquad}\n",
"r1 1 4\nr2 4 5\nc1 2 4\nc2 1 5\nU 2 3 5 3\nf 1 3 2 3\np g21\nend\n\n",

"{circuit 18.\n",
"Vo/Vi = k*(s^2 +1)/(s^2 + d*s +1)\n",
"biquad}\n",
"{note twin tee has fixed proportions\n",
"produced by repeated symbolic values.\n",
"the result here has a common uncancelled factor in the numerator\n",
"and the denominator}\n"
"r1 1 4\nr1 4 5\nr1 6 2\nr1 6 2\nc1 1 6\nc1 6 5\nc1 4 2\nc1 4 2\nU 2 3 5 3\nf 1 3 2 3\np g21\nend\n\n",


      "\n{examples to show parameter equivalences:}\n",
      "z11 1 3\nz22 2 4\nrm12 3 5 6 7\nrm21 4 6 5 7\nf 1 7 2 7\nend\n",
      "\ny11 1 3\ny22 2 4\ngm21 2 4 1 3\ngm12 1 3 2 4\ngx 3 4\nf 1 3 2 4\nend\n",
      "\ny11 1 3\ny22 2 3\ngm21 2 3 1 3\ngm12 1 3 2 3\nf 1 3 2 3\nend\n",
      "\n{deboo gyrator examples}\nr1 4 7\nr1 3 4\nr1 5 6\nr1 3 6\ng1 1 3\ng1 1 2\ng1 5 2\no 3 7 1 4\no 5 7 2 6\nf 1 7 2 7\np z11 p  z12 p z21 p z22\np y11\np y12\np y21\np y22\nend\n",
      "\nc1 1 7\nc2 2 7\nr1 4 7\nr1 3 4\nr1 5 6\nr1 3 6\ng1 1 3\ng1 1 2\ng1 5 2\no 3 7 1 4\no 5 7 2 6\nf 1 7 2 7\np z11\np  z22\nend\n",
      "\n{compare to native gyrator}\ngy 1 7 2 7\nf 1 7 2 7\np z11 p z12 p z21 p z22\np y11\np y12\np y21\np y22\nend\n\n",
      "\nc1 1 7\nc2 2 7\ngy 1 7 2 7\nf 1 7 2 7\np z11 p z22\nend\n",
      "\n{examples to show ambiguity due to the nullor model of opamps}\n",
      "{ideal opamp, positive feedback}\n",
      "opamp 2 0 3 0\nrin 1 3\nrf 2 3\nf 1 0 2 0\np g21\nend\n\n",
      "{ideal opamp, negative feedback}\n",
      "opamp 2 0 0 3\nrin 1 3\nrf 2 3\nf 1 0 2 0\np g21\nend\n\n",
      "{vcvs, positive feedback}\n",
      "uvcvs 2 0 3 0\nrin 1 3\nrf 2 3\nf 1 0 2 0\np g21\nend\n\n",
      "{vcvs, negative feedback}\n",
      "uvcvs 2 0 0 3\nrin 1 3\nrf 2 3\nf 1 0 2 0\np g21\nend\n\n",
      "{vcis, negative feedback}\n",
      "gmvcis 2 0 3 0\nrin 1 3\nrf 2 3\nf 1 0 2 0\np g21\nend\n\n",
      "{vcis, positive feedback}\n",
      "gmvcis 2 0 0 3\nrin 1 3\nrf 2 3\nf 1 0 2 0\np g21\nend\n\n",
      "\n{example demonstrating numeric value substitution}\n",
      "{from tuttle, v.1,1958, pp343-344}\n",
      "f 1 2 3 4\n",
      "p z11\n",
      "c1 1 3 v 78.56\n",
      "c2 3 4 v 11.42\n",
      "c3 4 2 v 1.111\n",
      "r1 3 4 v 0.0268\n",
      "r2 4 2 v 0.03446\n",
      "end\n",
      "\nf 1 2 3 4\n",
      "p z11\n",
      "c1 1 3 v 78.56\n",
      "c2 3 4 {v 11.42}\n",
      "c3 4 2 v 1.111\n",
      "r1 3 4 v 0.0268\n",
      "r2 4 2 v 0.03446\n",
      "end\n",
      "\nf 1 2 3 4\n",
      "p z11\n",
      "c1 1 3 {v 78.56}\n",
      "c2 3 4 {v 11.42}\n",
      "c3 4 2 {v 1.111}\n",
      "r1 3 4 {v 0.0268}\n",
      "r2 4 2 {v 0.03446}\n",
      "end\n",
      };


      for(i=0;i<(sizeof(help1)/sizeof(char *));i++)
       printf ("%s",help1[i]);

      printf ("branch name list %d\n", emax-(s_active+s_passive));
      printf ("node list %d\n", nmax-s_nodes);
      printf ("passive edges %d\n", pmax-s_passive);
      printf ("active edges %d\n", actmax-s_active);

      for(i=0;i<(sizeof(help2)/sizeof(char *));i++)
       printf ("%s",help2[i]);

      for(i=0;i<(sizeof(function_table)/sizeof(f_tab));i++)
      {
        evfn = function_table[i].fn_no;
        if((evfn >=12) && (evfn <= 19))
          continue; /*bypass s and F parameters*/

        printf(" %s%1d%1d = %s%1d/%s%1d for %s%1d = 0; topological formula: %c%s/%s\n",
          function_table[i].fn_name,
          port_node[function_table[i].fn_p1p] ,
          port_node[function_table[i].fn_p2p] ,
          function_table[i].var1_name,
          port_node[function_table[i].var1_p1p] ,
          function_table[i].var2_name,
          port_node[function_table[i].var2_p1p] ,
          function_table[i].var3_name,
          port_node[function_table[i].var3_p1p] ,
          (function_table[i].sign < 0)?'-':' ',
          topo_names[function_table[i].num_form] ,
          topo_names[function_table[i].denom_form] );

      };

      for(i=0;i<(sizeof(copyright_notice)/sizeof(char *));i++)
       printf ("%s",copyright_notice[i]);

      for(i=0;i<(sizeof(help3)/sizeof(char *));i++)
       printf ("%s",help3[i]);




}


/* notice that embedded comments are printed before the string or int is printed:

st{embedded comment}ring,  shows up at the input echo output as {embedded comment}string, 

or 2{embedded comment}3, is {embedded comment}23, 

other embedded characters in the middle in getints just disappear 1.2  is 12 etc. */

int getstr(int *len, int lim, char *str)
{
int open_brackets;

*len = 0;
*str = 0;
open_brackets = 0;

if (ch != 0) printf("%c",ch);

for(;;){
 if (scanf("%c",&ch) == EOF) {if (*len > 0)return(0); else return(EOF);};

 if (ch == '{' ) open_brackets++;
 if (open_brackets > 0) {
  printf("%c",ch);
  if (ch == '}' ) open_brackets--;
  continue;
  };


 switch (ch){
  case '\n':
  case '\r':
  case '\t':
  case ' ':
  case ',':
  case ';':
            if (*len > 0){
             return(0);
             }
             else
             printf("%c",ch);
             break;
  default:
           if (lim <= 1)
            return(0);
           (*len)++;
           *str = ch;
           str++;
           *str = 0;
           lim--;
           break;
 };


};


}

int getint(int *val)
{
int start,open_brackets;

*val = 0;
start = 0;

open_brackets = 0;

if (ch != 0) printf("%c",ch);


for(;;){
 if (scanf("%c",&ch) == EOF) {if (start != 0)return(0); else return(EOF);};

 if (ch == '{' ) open_brackets++;
 if (open_brackets > 0) {
  printf("%c",ch);
  if (ch == '}' ) open_brackets--;
  continue;
  };

 switch (ch){
  case '\n':
  case '\r':
  case '\t':
  case ' ':
  case ',':
  case ';':
            if (start != 0){
             return(0);
             }
             else
             printf("%c",ch);
             break;
  case '0':
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '9':
           start = 1;
           *val = ((*val) * 10) + (ch - '0');
           break;
  default:
           break;
 };


};



}

int getdouble(double *val)
{
int start,open_brackets;
int scale = -1;
int characteristic = 0;
int exp = 0;
double factor,sign = 1.0;

*val = 0;
start = 0;

open_brackets = 0;

if (ch != 0) printf("%c",ch);


for(;;){
 if (scanf("%c",&ch) == EOF) {if (start != 0)return(0); else return(EOF);};

 if (ch == '{' ) open_brackets++;
 if (open_brackets > 0) {
  printf("%c",ch);
  if (ch == '}' ) open_brackets--;
  continue;
  };

 switch (ch){
  case '\n':
  case '\r':
  case '\t':
  case ' ':
  case ',':
  case ';':
            if (start != 0){
             if (scale == -1) scale = 0;
             characteristic = (exp * characteristic) - scale;
             if (characteristic < 0)
               {characteristic = -characteristic; factor = 0.1;}
              else
               {factor = 10.0;};
             for(exp = 0; exp < characteristic; exp++){*val = (*val) * factor;};
             *val = (*val) * sign;
             return(0);
             }
             else
             printf("%c",ch);
             break;
  case '0':
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '9':
           start = 1;
           if (exp != 0){
            characteristic = (characteristic * 10) + (ch - '0');
           } else {
           *val = ((*val) * 10.0) + (ch - '0');
           if (scale >= 0) scale++;
           };
           break;
  case '.':
           if ( (scale >= 0) || (exp != 0) ) break;
           scale = 0;
           break;
  case 'e':
  case 'E':
           exp = 1;
           break;
  case '-':
           if (exp == 1) exp = -1;
           if (start == 0){
            start = 1;
            sign = -1.0;
           };
           break;
  default:
           break;
 };


};



}
/* convenient input node relabeling */
int 
nodemap (int node)
{
  int i;

  if (node > 999) 
  {
    printf ("node label %d must be less than 1000\n", node);
    return (1);
  }
  ;

  for (i = 1; i <= n; i++) 
  {
    if (nodelist[i] == node)
      return (i);
  }
  ;
  if (n >= (nmax-s_nodes)) 
  {
    printf ("number of nodes(= %d) exceeds limit of %d\n or improper input applied\n", n, nmax - 1 - s_nodes);
    return (1);
  }
  ;
  n++;
  nodelist[n] = node;
  return (n);
}
int 
internal_node (void)
{
  if (n >= nmax) 
  {
    printf ("number of nodes(= %d) exceeds limit of %d\n or improper input applied\n", n, nmax - 1);
    return (1);
  }
  ;

  n++;
  int_node++;
  nodelist[n] = int_node;
  return (n);
}

/*these arrays are an attempt to reduce the lookup work done in
  the busiest loops */

void eval_maps(void)
{
  int i,j, ch0;

/* this maps identical branch labels to the first instance. */
  for (j = 1; j <= branches; j++) 
  {
    branchmap[j] = getsym (j);
  }
  ;

  for (i = 0; i < maxsym; i++)
    branchtag[i] = 0;

  impedancecount = 0;


  for (i = 1; i <= branches; i++) 
  {

    j = branchmap[i] /*getsym (i, (e + a))*/;
    ch0 = branch[j][0];

/* powermap is used to evaluate the exponent of s for the
   mixed unit admittance products */
    if ((ch0 == 'c') ||
      (ch0 == 'C') ||
      (ch0 == 'l') ||
      (ch0 == 'L') ||
      (ch0 == 'm') ||
      (ch0 == 'M'))
      powermap[i] = 1;
    else
      powermap[i] = 0;

/* fmap allows certain factors of the admittance product to be specially
 treated */
    if (ch0 == 'a')
      fmap[i] = UNITY;
    else
    {
      if ((ch0 == 'o') ||
        (ch0 == 'O'))
        fmap[i] = OP;
      else
        fmap[i] = 0;
    }
    ;
/* impedancelist is used to correct the mixed unit
   products (resistance and reactance vs. conductance
 and susceptance) to proper admittance (just conductance and susceptance) only products */
    if (i > j)
      continue;
    if ((ch0 == 'r') ||
      (ch0 == 'R') ||
      (ch0 == 'z') ||
      (ch0 == 'Z') ||
      (ch0 == 'l') ||
      (ch0 == 'L') ||
      (ch0 == 'm') ||
      (ch0 == 'M')) 
    {
      impedancelist[impedancecount] = j;
      impedancecount++;
    }
    ;
  }
  ;

}
/* finds the first instance of potentially multiple branch names */
int 
getsym (int i)
{

  int j, k, l, m;


  k = 1;
  while (k <= branches) 
  {
    m = 0;
    while ((branch[k][m] == branch[i][m]) && (branch[i][m] != 0))
      m++;
    if (branch[k][m] == branch[i][m]) 
    {
      return k;
    }
    ;
    k++;
  }
  ;
  printf ("can't find %s <=> branch[%d] in %d branches\n", &branch[i][0], i, branches);
  return i;

}

/* controls the evaluation of the required topological functions;
   deals with the augmentation for scattering parameters;
   combines the evaluated topological function numerator and denominator
   for the desired two-port parameter evaluations */

void analyze_network()
{
  int i;
#ifdef COMBINED_TREES
  int fun_group;
#else
  int funs_evaluated;
#endif
  int port_node[4];

/* numerator and denominator indicators
   #define V     0
   #define wijkl 1
   #define wijij 2
   #define wklkl 3
   #define wklij 4
   #define uijkl 5
 */

/*
   transfer ratios:
   Vkl/Vij = wijkl/wijij
   Ikl/Iij = -wijkl/wklkl
   open circuit admittance functions:
   Iij/Vij = V/wijij
   Ikl/Vij = V/wklij
   Iij/Vkl = V/wijkl
   Ikl/Vkl = V/wklkl
   short circuit admittance functions:
   Iij/Vij = wklkl/uijkl
   Iij/Vkl = -wklij/uijkl
   Ikl/Vij = -wijkl/uijkl
   Ikl/Vkl = wijij/uijkl
 */

  port_node[0] = ni1;
  port_node[1] = nj1;
  port_node[2] = nk1;
  port_node[3] = nl1;


  for(i=0;i<6;i++)
    topological_fun[i] = 0;

#ifdef COMBINED_TREES
   fun_group = 0;
#else
   funs_evaluated = 0;
#endif

  for(i=0;i<(sizeof(function_table)/sizeof(f_tab));i++)
  {


    evfn = function_table[i].fn_no;

    if(par_list[i] == 0)
      continue;

    if((evfn < 12) || (evfn > 19))
    {
#ifdef COMBINED_TREES

      topological_fun[function_table[i].num_form] = 1;

      if((function_table[i].num_form == wijkl) ||
         (function_table[i].num_form == wklij) )
       fun_group |= 2;
      else
       fun_group |= 1;

      topological_fun[function_table[i].denom_form] = 1;

      if((function_table[i].denom_form == wijkl) ||
         (function_table[i].denom_form == wklij) )
       fun_group |= 2;
      else
       fun_group |= 1;

#else

      if(topological_fun[function_table[i].num_form] == 0)
      {
        funs_evaluated++;
        topological_fun[function_table[i].num_form] = 1;
        ev_fn (function_table[i].num_form);
      }
      ;
      if(topological_fun[function_table[i].denom_form] == 0)
      {
        funs_evaluated++;
        topological_fun[function_table[i].denom_form] = 1;
        ev_fn (function_table[i].denom_form);
      }
      ;
#endif
    }
    ;

  }
  ;
#ifdef COMBINED_TREES

   if((fun_group &0x1) != 0)
     ev_fn(0);

   if((fun_group &0x2) != 0)
     ev_fn(4);

   
   if(fun_group > 0) {

   for(i=0;i<(sizeof(function_table)/sizeof(f_tab));i++){

      evfn = function_table[i].fn_no;

      if(par_list[i] == 0) continue;


      if( ((evfn < 12) || (evfn >  19)) &&
         (topological_fun[function_table[i].num_form] != 0) &&
         (topological_fun[function_table[i].denom_form] != 0)){
         printf (" Function:\n %s( %d, %d; %d, %d) = %s( %d, %d)/%s( %d, %d) for %s( %d, %d)=0\n",
           function_table[i].fn_name,
           port_node[function_table[i].fn_p1p] ,port_node[function_table[i].fn_p1m],
           port_node[function_table[i].fn_p2p] ,port_node[function_table[i].fn_p2m] ,
           function_table[i].var1_name,
           port_node[function_table[i].var1_p1p] ,port_node[function_table[i].var1_p1m] ,
           function_table[i].var2_name,
           port_node[function_table[i].var2_p1p] ,port_node[function_table[i].var2_p1m] ,
           function_table[i].var3_name,
           port_node[function_table[i].var3_p1p] ,port_node[function_table[i].var3_p1m] );


         print_treebuf (function_table[i].sign, function_table[i].num_form, function_table[i].denom_form);
      };
   };


    release_topo_data();

  };


#else
  if(funs_evaluated > 0) 
  {

    for(i=0;i<(sizeof(function_table)/sizeof(f_tab));i++)
    {

      evfn = function_table[i].fn_no;

      if(par_list[i] == 0)
        continue;

      if( ((evfn < 12) || (evfn >  19)) &&
        (topological_fun[function_table[i].num_form] != 0) &&
        (topological_fun[function_table[i].denom_form] != 0))
      {
        printf (" Function:\n %s( %d, %d; %d, %d) = %s( %d, %d)/%s( %d, %d) for %s( %d, %d)=0\n",
          function_table[i].fn_name,
          port_node[function_table[i].fn_p1p] ,port_node[function_table[i].fn_p1m],
          port_node[function_table[i].fn_p2p] ,port_node[function_table[i].fn_p2m] ,
          function_table[i].var1_name,
          port_node[function_table[i].var1_p1p] ,port_node[function_table[i].var1_p1m] ,
          function_table[i].var2_name,
          port_node[function_table[i].var2_p1p] ,port_node[function_table[i].var2_p1m] ,
          function_table[i].var3_name,
          port_node[function_table[i].var3_p1p] ,port_node[function_table[i].var3_p1m] );


        print_treebuf (function_table[i].sign, function_table[i].num_form, function_table[i].denom_form);
      }
      ;
    }
    ;


    release_topo_data();

  }
  ;
#endif



  for(i=0;i<6;i++)
    topological_fun[i] = 0;

#ifdef COMBINED_TREES
   fun_group = 0;

   for(i=0;i<(sizeof(function_table)/sizeof(f_tab));i++){

      if(par_list[i] == 0) continue;

      evfn = function_table[i].fn_no;

      if((evfn >= 12) && (evfn <= 19)){

        if(fun_group == 0) embed_network_for_s_parameters();

      topological_fun[function_table[i].num_form] = 1;

      if((function_table[i].num_form == wijkl) ||
         (function_table[i].num_form == wklij) )
       fun_group |= 2;
      else
       fun_group |= 1;

      topological_fun[function_table[i].denom_form] = 1;

      if((function_table[i].denom_form == wijkl) ||
         (function_table[i].denom_form == wklij) )
       fun_group |= 2;
      else
       fun_group |= 1;

      };

   };

   if((fun_group &0x1) != 0)
     ev_fn(0);

   if((fun_group &0x2) != 0)
     ev_fn(4);

   if(fun_group > 0) {

   for(i=0;i<(sizeof(function_table)/sizeof(f_tab));i++){

      evfn = function_table[i].fn_no;

      if(par_list[i] == 0) continue;

      if( ((evfn >= 12) && (evfn <=  19)) &&
         (topological_fun[function_table[i].num_form] != 0) &&
         (topological_fun[function_table[i].denom_form] != 0)){
         printf (" Function:\n %s( %d, %d; %d, %d) for %s normalization\n",
           function_table[i].fn_name,
           port_node[function_table[i].fn_p1p] ,port_node[function_table[i].fn_p1m],
           port_node[function_table[i].fn_p2p] ,port_node[function_table[i].fn_p2m] ,
           (normalization == 0)?"Y0":(normalization == 1)?"Z0":"unity");
         print_treebuf (function_table[i].sign, function_table[i].num_form, function_table[i].denom_form);
      };
   };

    release_topo_data();


  };


#else

  funs_evaluated = 0;

  for(i=0;i<(sizeof(function_table)/sizeof(f_tab));i++)
  {

    if(par_list[i] == 0)
      continue;

    evfn = function_table[i].fn_no;

    if((evfn >= 12) && (evfn <= 19))
    {

      if(funs_evaluated == 0)
        embed_network_for_s_parameters();

      if(topological_fun[function_table[i].num_form] == 0)
      {
        funs_evaluated++;
        topological_fun[function_table[i].num_form] = 1;
        ev_fn (function_table[i].num_form);
      }
      ;
      if(topological_fun[function_table[i].denom_form] == 0)
      {
        funs_evaluated++;
        topological_fun[function_table[i].denom_form] = 1;
        ev_fn (function_table[i].denom_form);
      }
      ;

    }
    ;

  }
  ;

  if(funs_evaluated > 0) 
  {

    for(i=0;i<(sizeof(function_table)/sizeof(f_tab));i++)
    {

      evfn = function_table[i].fn_no;

      if(par_list[i] == 0)
        continue;

      if( ((evfn >= 12) && (evfn <=  19)) &&
        (topological_fun[function_table[i].num_form] != 0) &&
        (topological_fun[function_table[i].denom_form] != 0))
      {
        printf (" Function:\n %s( %d, %d; %d, %d) for %s normalization\n",
          function_table[i].fn_name,
          port_node[function_table[i].fn_p1p] ,port_node[function_table[i].fn_p1m],
          port_node[function_table[i].fn_p2p] ,port_node[function_table[i].fn_p2m] ,
          (normalization == 0)?"Y0":(normalization == 1)?"Z0":
          "unity");
        print_treebuf (function_table[i].sign, function_table[i].num_form, function_table[i].denom_form);
      }
      ;
    }
    ;

    release_topo_data();


  }
  ;
#endif

}

/* the input edge lists are copied to the v- and i- graphs for
   spanning tree matching; this requires adding additional edges, F1 and F2
   to indicate port conditions for some topological formulas;
   the active elements may have different nodes in the two graphs 
   denoting their own port relationships */
void 
#ifdef COMBINED_TREES
evaluate_fn (int e, int a, int r, int toposet)
#else
evaluate_fn (int e, int a, int r, int topofun)
#endif
{
  int i, j, k, ch0, rc, passive_edges, minty_nodes, actual_nodes, active_edges,
    tree_form;
  int total_edges;

  evfn++;


#ifndef COMBINED_TREES

  if ((test & (PRINT_STATS | RAW_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS | DEBUG_128)) != 0) 
  {
    printf("Topological formula: (%s, %1d-trees) \n",
      topo_names[topofun],(topofun ==0)?1:((topofun == 5)?3:
      2));
  }
  ;
#endif

  minty_nodes = r + 1;
  actual_nodes = r;
  passive_edges = e;
  dtag = 0;

#ifdef COMBINED_TREES
   for(i=0;i<=3;i++)
    if(topomap[i + toposet] >= 0)
      init(VARLIST((topomap[i + toposet])));
#endif

/* the preferred ordering for the list of graph edges is: handle edge,
   port condition edges F1 and/or F2, opamp edges, passive edges, then
   the remaining active edges. this ordering helps truncate the spanning tree
   search space early in order to reduce the number of spanning trees that
   need to be evaluated in function minty */
  total_edges = 1;
  d0[1] = p0[1];
  dv1[1] = p1[1];
  dv2[1] = p2[1];
  di1[1] = p1[1];
  di2[1] = p2[1];
#ifdef COMBINED_TREES
   fmap[total_edges] = HANDLE_BRANCH;
#else
   fmap[total_edges] = 0;
#endif


#ifdef COMBINED_TREES

      total_edges++;
      dv1[total_edges] = ni;
      dv2[total_edges] = nj;
      if(toposet == 0){
       di1[total_edges] = ni;
       di2[total_edges] = nj;}
      else{
       di1[total_edges] = nk;
       di2[total_edges] = nl;};
      d0[total_edges] = branches + 1;
      branch[branches + 1][0] = 'f';
      branch[branches + 1][1] = '1';
      branch[branches + 1][2] = 0;
      fmap[d0[total_edges]] = F1;
      total_edges++;
      dv1[total_edges] = nk;
      dv2[total_edges] = nl;
      if(toposet == 0){
       di1[total_edges] =  nk;
       di2[total_edges] =  nl;}
      else{
       di1[total_edges] = ni;
       di2[total_edges] = nj;};
      branch[branches + 2][0] = 'f';
      branch[branches + 2][1] = '2';
      branch[branches + 2][2] = 0;
      d0[total_edges] = branches + 2;
      fmap[d0[total_edges]] = F2;

#else

  init(VARLIST(topofun));
/* port condition indication edges added to the analysis graphs here */
  switch (topofun) 
  {
  case V:
    tree_form = ONETREES;
    break;

  case wijkl:
    tree_form = TWOTREES;
    total_edges++;
    dv1[total_edges] = ni;
    dv2[total_edges] = nj;
    di1[total_edges] =  nk;
    di2[total_edges] =  nl;
    branch[branches + 1][0] = 'f';
    branch[branches + 1][1] = 0;
    d0[total_edges] = branches + 1;
    fmap[d0[total_edges]] = F1;
    break;

  case wijij:
    tree_form = TWOTREES;
    total_edges++;
    dv1[total_edges] = ni;
    dv2[total_edges] = nj;
    di1[total_edges] = ni;
    di2[total_edges] = nj;
    d0[total_edges] = branches + 1;
    branch[branches + 1][0] = 'f';
    branch[branches + 1][1] = 0;
    fmap[d0[total_edges]] = F1;
    break;

  case wklkl:
    tree_form = TWOTREES;
    total_edges++;
    dv1[total_edges] = nk;
    dv2[total_edges] = nl;
    di1[total_edges] = nk;
    di2[total_edges] = nl;
    d0[total_edges] = branches + 1;
    branch[branches + 1][0] = 'f';
    branch[branches + 1][1] = 0;
    fmap[d0[total_edges]] = F1;
    break;

  case wklij:
    tree_form = TWOTREES;
    total_edges++;
    dv1[total_edges] = nk;
    dv2[total_edges] = nl;
    di1[total_edges] =  ni;
    di2[total_edges] =  nj;
    branch[branches + 1][0] = 'f';
    branch[branches + 1][1] = 0;
    d0[total_edges] = branches + 1;
    fmap[d0[total_edges]] = F1;
    break;

  case uijkl:
    tree_form = THREETREES;
    total_edges++;
    dv1[total_edges] = ni;
    dv2[total_edges] = nj;
    di1[total_edges] = ni;
    di2[total_edges] = nj;
    d0[total_edges] = branches + 1;
    branch[branches + 1][0] = 'f';
    branch[branches + 1][1] = '1';
    branch[branches + 1][2] = 0;
    fmap[d0[total_edges]] = F1;
    total_edges++;
    dv1[total_edges] = nk;
    dv2[total_edges] = nl;
    di1[total_edges] =  nk;
    di2[total_edges] =  nl;
    branch[branches + 2][0] = 'f';
    branch[branches + 2][1] = '2';
    branch[branches + 2][2] = 0;
    d0[total_edges] = branches + 2;
    fmap[d0[total_edges]] = F2;
    break;

  }
  ;
#endif

/* opamp edges added next */
  for (i = 1; i <= a; i++) 
  {
    if(fmap[a0[i]] == OP)
    {
      total_edges++;
      d0[total_edges] = a0[i];
      dv1[total_edges] = a1[i];
      dv2[total_edges] = a2[i];
      di1[total_edges] = a3[i];
      di2[total_edges] = a4[i];
    }
    ;
  }
  ;





/* passive edges added to graphs */
  for (i = 2; i <= e; i++) 
  {
    total_edges++;
    d0[total_edges] = p0[i];
    dv1[total_edges] = p1[i];
    dv2[total_edges] = p2[i];
    di1[total_edges] = p1[i];
    di2[total_edges] = p2[i];
  }
  ;

/* remaining active edges added to v- and i- graphs */
  for (i = 1; i <= a; i++) 
  {
    if(fmap[a0[i]] != OP)
    {
      total_edges++;
      d0[total_edges] = a0[i];
      dv1[total_edges] = a1[i];
      dv2[total_edges] = a2[i];
      di1[total_edges] = a3[i];
      di2[total_edges] = a4[i];
    }
    ;
  }
  ;



/* strength reduction to reduce work in busiest loops */
  for (i = 1; i <= total_edges; i++)
  {
    dmap[i] = fmap[d0[i]];
    dpowermap[i] = powermap[d0[i]];
    dbranchmap[i] = branchmap[d0[i]];
  }
  ;

/* evaluate all qualified spanning trees */
#ifdef COMBINED_TREES
/* up to four topological formulas are evaluated together; they are partitioned
   by the particular combination of F1 and F2 edges in each common spanning tree */
   if ((rc = product (minty_nodes, total_edges, actual_nodes,
     tree_form, e + a, toposet)) != 0) {
      printf ("insufficient memory to buffer common trees; remainder of function evaluation skipped\n");
      return;
   };

#else
/* a single topological function at a time is evaluated */
  if ((rc = product (minty_nodes, total_edges, actual_nodes,
    tree_form, e + a, topofun)) != 0) 
  {
    printf ("insufficient memory to buffer common trees; remainder of function evaluation skipped\n");
    return;
  }
  ;
#endif

  if ((test & (PRINT_STATS)) != 0)
    printf ("opampcnt= %d\n", opampcnt);


}

/* the original input edge lists are augmented by an active network at
   each port that allows scattering parameters to be evaluated using the existing
   topological function evaluation framework */
void embed_network_for_s_parameters(void)
{
/* -1<=>nk,-2<=>nl,-3<=>nj,-4<=>ni */
#define nodesubs(nnn) ((nnn == -1)?(nksave):((nnn == -2)?(nl):((nnn == -3)?(nj):((nnn == -4)?(nisave):(new_nodes[nnn])))))

  int nisave, nksave;
  int i, j, k;
  int new_nodes[10];
/* the embedding is driven by these tables to reduce the complexity of inline code */
  int opamp_list[][4] = 
    {
    {
      2, -3, 1, 2
    }
    ,
    {
      3, -3, -4, 4
    }
    ,
    {
      6, -2, 5, 6
    }
    ,
    {
      7, -2, -1, 8
    }
    ,
    {
      0, 0, 0, 0
    }
    ,
  }
  ;

/* vcis unused in embedding circuit */
  int vcis_list[][4] = 
    {
    {
      0, 0, 0, 0
    }
    ,
  }
  ;

  int r_list[][2] = 
    {
    {
      1, 3
    }
    ,
    {
      4, 3
    }
    ,
    {
      4, -3
    }
    ,
    {
      5, 7
    }
    ,
    {
      7, 8
    }
    ,
    {
      8, -2
    }
    ,
    {
      0, 0
    }
    ,
  }
  ;
  int ref_list[][2] = 
    {
    {
      2, -4
    }
    ,
    {
      6, -1
    }
    ,
    {
      0, 0
    }
    ,
  }
  ;

  printf("entered s-param\n");

  nisave = ni;
  nksave = nk;

  n--; /* back out dummy node for dummy edge */

  for(i=1;i<=8;i++) {
     if (((i == 2) && (normalnet1 != 0) && (normalnet2 != 0)) ||
         ((i == 6) && (normalnet3 != 0) && (normalnet4 != 0)) ) {
/* connect one end of the normalizing networks to the augmentation networks*/

          new_nodes[i] = (i ==  2)?normalnet1:normalnet3;

     } else {
        new_nodes[i] = internal_node();
     }
  }
  ni = nodesubs(1);
  nk = nodesubs(5);

  for(i=0;opamp_list[i][0] != 0;i++)
  {

    opampcnt++;
    branches++;
    a++;
    a0[a] = branches;
    a1[a] = nodesubs(opamp_list[i][0]);
    a2[a] = nodesubs(opamp_list[i][1]);
    a3[a] = nodesubs(opamp_list[i][2]);
    a4[a] = nodesubs(opamp_list[i][3]);
    branch[branches][0] = 'o';
    branch[branches][1] = 0;


  }
  ;


  for(i=0;vcis_list[i][0] != 0;i++)
  {


    branches++;
    a++;
    a0[a] = branches;
    a1[a] = nodesubs(vcis_list[i][0]);
    a2[a] = nodesubs(vcis_list[i][1]);
    a3[a] = nodesubs(vcis_list[i][2]);
    a4[a] = nodesubs(vcis_list[i][3]);
    branch[branches][0] = 'a';
    branch[branches][1] = 0;


  }
  ;

  for(i=0;r_list[i][0] != 0;i++)
  {

    branches++;
    p++;
    p0[p] = branches;
    p1[p] = nodesubs(r_list[i][0]);
    p2[p] = nodesubs(r_list[i][1]);
    branch[branches][0] = 'a';
    branch[branches][1] = 0;


  }
  ;

  for(i=0;ref_list[i][0] != 0;i++)
  {
    if (((i == 0) && (normalnet1 != 0) && (normalnet2 != 0)) ||
        ((i == 1) && (normalnet3 != 0) && (normalnet4 != 0)) ) {
/*   use nullor (nullator and norator in parallel) element 'o' to link the normalization network to the remainder of the network*/
       branches++;
       a++;
       a0[a] = branches;
       a1[a] = (i == 0)?normalnet2:normalnet4;
       a2[a] = nodesubs(ref_list[i][1]);
       a3[a] = a1[a];
       a4[a] = a2[a];
       branch[branches][0] = 'o';
       branch[branches][1] = 0;

    } else {
       branches++;
       p++;
       p0[p] = branches;
       p1[p] = nodesubs(ref_list[i][0]);
       p2[p] = nodesubs(ref_list[i][1]);
       switch(normalization)
       {
       case 0:
         branch[branches][0] = 'Y';
         branch[branches][1] = '0';
         branch[branches][2] = 0;
         break;
       case 1:
         branch[branches][0] = 'Z';
         branch[branches][1] = '0';
         branch[branches][2] = 0;
         break;
       default:
         branch[branches][0] = 'a';
         branch[branches][1] = 0;
       }
       ;

    }


  }
  ;



/* dummy edge (handle) and extra node */
  p0[1] = 1;
  p1[1] = n + 1; /* update dummy node for this dummy edge */
  p2[1] = 1;
  branch[1][0] = 'x';
  branch[1][1] = 0;
  n++;

  e = p;
  r = n - 1;

  eval_maps();


  if ((test & INPUT_DUMP) != 0) 
  {
    printf ("Internal representation of data including generated elements and reassigned node#s:\n");
    printf ("this includes embedding circuitry for evaluating scattering parameters.\n");
    printf ("edge,name,node#'s...\n");
    for (i = 1, j = 1; i <= p; i++, j++) 
    {
      printf ("%3d %s %d %d\n", j, &branch[p0[i]][0], p1[i], p2[i]);
    }
    ;
    for (i = 1; i <= a; i++, j++) 
    {
      printf ("%3d %s %d %d %d %d\n", j, &branch[a0[i]][0], a1[i], a2[i], a3[i], a4[i]);
    }
    ;
    printf ("%3d f %d %d %d %d\n", j, ni, nj, nk, nl);
    printf ("internal node#  input node#:\n");
    for (i = 1; i <= n; i++)
      printf ("         %5d    %d\n", i, nodelist[i]);
    printf ("\nedge namelist: index, value\n");
    for (i = 1; i <= branches; i++)
      if ( value[i] != 0) {printf ("%3d, %s %g\n", i, &branch[i][0], value[i]);}
      else {printf ("%3d, %s\n", i, &branch[i][0]);};
    printf ("\n");
  }
  ;


}

/* each distinct topological formula has its own data structures to buffer
   the collections of sums of distinct admittance products-- they are allocated here */
void 
init (PARAMETERLIST)
{
  int i, j, k;
  int (*poly)[maxdegree], (*termlist)[maxsym + 2];
  double (*termvalue);

  *maxtermlist = maxterm * maxdegree;
  *maxpoly = maxterm;
  *ptermlist = (void *) malloc ((*maxtermlist) * sizeof (int) * (maxsym + 2));
  *ptermvalue = (void *) malloc ((*maxtermlist) * sizeof (double));
  *ppoly = (void *) malloc ((*maxpoly) * sizeof (int) * maxdegree);

  poly = *ppoly;
  termlist = *ptermlist;
  termvalue = *ptermvalue;

  for (j = 0; j < maxdegree; j++)
    for (k = 0; k < (*maxpoly); k++)
      poly[k][j] = 0;

  for (i = 0; i < (*maxtermlist); i++)
    {
      termvalue[i] = 0.0;
      for (j = 0; j <= (maxsym + 1); j++)
        termlist[i][j] = 0;
    };
  for (j = 0; j < maxdegree; j++)
    degterms[j] = 0;
  *toterms = 0;

}

void release_topo_data(void)
{
  int i;
  for(i=V;i<=uijkl;i++)
    if (topological_fun[i] !=0)
      release(VARLIST(i));

}


void
release (PARAMETERLIST)
{
  free (*ptermlist);
  free (*ppoly);
  free (*ptermvalue);

}

/* the complete collection of common spanning trees for a topological function is
   evaluated in this function. the sign of each spanning tree is evaluated, and the signed
   accumulation of these trees is deposited into the data structure
   for its topological function.  */
int
#ifdef COMBINED_TREES
/* this function deals with up to four related topological functions at a time instead of only one at a time */
product (int minty_nodes, int edges, int actual_nodes, int form, int br, int toposet)
#else
product (int minty_nodes, int edges, int actual_nodes, int form, int br, int topofun)
#endif
{
  static   int i, ii, j, k, m, nn, ncom, s, rc, sign, tmp, itreepermbuf[nmax], vtreepermbuf[nmax];
  static   int tree[nmax];
  static   int ch0;
  static   int r;


  if ((test & (PRINT_STATS | RAW_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS | DEBUG_128)) != 0) 
  {
#ifdef COMBINED_TREES
    printf("product:: minty_nodes:%d, edges:%d, actual_nodes:%d, form:%d, toposet:%d\n",
      minty_nodes, edges, actual_nodes, form, toposet);
#else
    printf("product:: minty_nodes:%d, edges:%d, actual_nodes:%d, form:%d, topofun:%d\n",
      minty_nodes, edges, actual_nodes, form, topofun);
#endif
  }
  ;

  ncom = 0;

  state = 0;

  mn = minty_nodes;
  me = edges;
  mr = actual_nodes;
#ifdef COMBINED_TREES
  mq = toposet;
#else
  mq = form;
#endif
  r = minty_nodes - 1;
  if ((test & (RAW_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS | DEBUG_128)) != 0) 
  {


    printf("i d0[i] dmap[i] fmap[d0[i]] dpowermap[i] powermap[d0[i]] dbranchmap[i] branchmap[d0[i]]\n");
    for (i = 1; i <= me; i++)
    {
      printf("%2d %2d %2d %2d %2d %2d %2d %2d\n",
        i,d0[i],dmap[i],fmap[d0[i]],
        dpowermap[i],powermap[d0[i]],
        dbranchmap[i],branchmap[d0[i]]);
    }
    ;


    printf("only %s-trees are accepted\n",(mq == TWOTREES)?"2":((mq == ONETREES)?"1":
      "3"));
    printf ("i-graph to generate i-trees:\nname:edge#,nameindex,node1,#node2#\n");
    for (i = 1; i <= me; i++)
      printf ("%5s: d0[%3d]=%3d, d1=%3d, d2=%3d\n", &branch[d0[i]][0], i, d0[i], di1[i], di2[i]);
    printf ("\n");
    printf ("\n");
    printf ("v-graph to check i-trees:\nname:edge#,nameindex,node1,#node2#\n")
    ;
    for (i = 1; i <= me; i++)
      printf ("%5s: d0[%3d]=%3d, d1=%3d, d2=%3d\n", &branch[d0[i]][0], i, d0[i], dv1[i], dv2[i]);
    printf ("\n");
    for (i = 1; i <= 1024; i++)
      printf(" ");
    printf ("\n");


  }
  ;
/* search for common trees only */
/* potentially, an efficiency improvement may result in inlining minty and treecheck with proper
   treatment of their internal variables */
  minty ();
  {
    m = 0;
    for (; (state == 1) ;minty ()) 
    {
/* each common spanning tree travels through this loop */
      ncom++;

      treecheck (r - 1, di1, di2, mtree, &itreepermbuf[0]);
      treecheck (r - 1, dv1, dv2, mtree, &vtreepermbuf[0]);

      if ((test & (TREE_PERM)) != 0) 
      {
        printf (" common tree edges:");
        for (i = 1; i < r; i++)
          printf ("%3d(%s) ", mtree[i],&branch[d0[mtree[i]]][0]);
        printf ("\n");

        printf ("v-tree permutation: ");
        for (i = 2; i <= r; i++) 
        {
          printf ("%3d(%s) ", vtreepermbuf[i],&branch[d0[ABS(vtreepermbuf[i])]][0]);
        }
        ;
        printf ("\n");
        printf ("i-tree permutation: ");
        for (i = 2; i <= r; i++) 
        {
          printf ("%3d(%s) ", itreepermbuf[i],&branch[d0[ABS(itreepermbuf[i])]][0]);
        }
        ;
        printf ("\n");
      }
/* capture the principal node component of the sign computation */
      sign = 1;
      for (i = 2; i <= r; i++) 
      {
        if (vtreepermbuf[i] < 0) 
        {
          sign = sign * (-1);
          vtreepermbuf[i] = (-vtreepermbuf[i]);
        }
        ;
        if (itreepermbuf[i] < 0) 
        {
          sign = sign * (-1);
          itreepermbuf[i] = (-itreepermbuf[i]);
        }
        ;
      }
      ;
/* capture the permutation component of the sign computation */
      for (i = 2; i <= r; i++) 
      {
        {
          if (vtreepermbuf[i] != itreepermbuf[i]) 
          {
            sign = sign * (-1);
            for (j = i + 1; j <= r; j++) 
            {
              if (vtreepermbuf[j] == itreepermbuf[i]) 
              {
                tmp = vtreepermbuf[j];
                vtreepermbuf[j] = vtreepermbuf[i];
                vtreepermbuf[i] = tmp;
                break;
              }
              ;
            }
            ;
          }
          ;
        }
        ;
      }
      ;
      if ((test & (TREE_PERM)) != 0)
        printf ("resulting sign= %2d\n", sign);


      s = 0;
      for (i = 1, ii = 0; i < r; i++) 
      {
/* evaluate the mixed unit power of s in order to classify the admittance products */
        s = s + dpowermap[mtree[i]];

/* drop certain edges after the sign is established:
   F1, F2, opamps, and 'a'-- unity elements */
        if (dmap[mtree[i]] != 0)
          continue;

        tree[ii] = mtree[i];
        ii++;
      }
      ;


      if ((test & (RAW_TREES)) != 0) 
      {
        printf (" accepted tree edges:");
        for (i = 0; i < ii; i++)
          printf ("%3d,%3d(%s) ", tree[i],d0[tree[i]],&branch[d0[tree[i]]][0]);
        printf("\n");
      }
      ;

      {
#ifdef COMBINED_TREES
/* topomap accomplishes the separation of the comingled topological function evaluations for the multiple
   evaluation scheme */
            if ((rc = getexpr (sign, s, ii, tree, br, VARLIST((topomap[mtree[0]])))) != 0) {
               release (VARLIST((topomap[mtree[0]])));
               topological_fun[topomap[mtree[0]]] = 0;
               printf ("insufficient memory for combining admittance terms\n");
               printf("raw i-trees  evaluated:%9d\n",nt);
               printf("qualified common trees:%9d\n",ncom);
    printf("raw trees: %d, potential trees: %d, common trees: %d\n",raw_nt,usable_nt,common_nt);
               return (rc);
            };
#else
        if ((rc = getexpr (sign, s, ii, tree, br, VARLIST(topofun))) != 0) 
        {
          release (VARLIST(topofun));
          topological_fun[topofun] = 0;
          printf ("insufficient memory for combining admittance terms\n");
          printf("raw i-trees  evaluated:%9d\n",nt);
          printf("qualified common trees:%9d\n",ncom);
          return (rc);
        }
        ;
#endif

      }
      ;

    }
    ;
  }
  ;

  if ((test & (PRINT_STATS)) != 0) 
  {
    printf("raw i-trees  evaluated:%9d\n",nt);
    printf("qualified common trees:%9d\n",ncom);
#ifdef COMBINED_TREES
    printf("raw trees: %d, potential trees: %d, common trees: %d\n",raw_nt,usable_nt,common_nt);
#endif
  }
  ;
  return (0);
}

/* this is the busiest function, since it has up to n^(n-2) complexity
   this algorithm was initially the c translation of Chan's implementation
   of Minty's algorithm that was published in Lin's book in the references.
   Two different changes have been made to the original: the first, due to an
   unpublished communication from Dr Benedykt S. Rodanski, of University of Technology, Sydney,
   checks each i-tree against the v-graph to test for tree commonality. This avoids two parallel searches,
   one in the i-graph, the other in the v-graph, for the common (edge set matching) trees. The second change
   is, I believe original to my implementation: the search space built up by Minty's algorithm branches by
   allowing both inclusion and exclusion for each edge in the graph. The realization that certain edges are
   essential for acceptable spanning trees means that the exclusion side subspace may be truncated and never
   explored for all their defective spanning trees. it also eliminates the need to check each spanning tree
   found to be sure it contains all the essential branches. the initial testing of minty's algorithm showed
   it must always select the first edge in the edge list for any spanning tree; this means that edges or loops
   parallel to it may not be properly found in spanning trees; the fix (as a band-aid) is to always include a
   (dummy) pendent edge first on the list, with a (dummy) additional node, to serve as this 'handle' for the 
   spanning tree. this edge is dropped from the resulting spanning tree. this is really an indication of a bug,
   which may have developed in the translation fron fortran to c. probably a very rigorous analysis of the
   search starting condition may remove this hack. 
   */
void 
minty (void)
{				/* avoid parameter passing if possible */
  static    int vd1, vd2, i, j, k, l, m, f0, op;
  static    int v[nmax], edge[emax];

  static   int stack[nmax], tag[nmax];
  static   int depth, done, root_node, princip_node;

  switch (state) 
  {
  case 0:
    goto l10;
  case 1:
    goto l25;
  default:
    state = 2;
    return;
  }
  ;
l10:
  state = 1;
  oprej = 0;



  for (k = 1; k <= me; k++)
    edge[k] = 1;
  for (k = 1; k <= mn; k++)
    v[k] = 0;
  it = 0;
  i = 1;
  nt = 0;
#ifdef COMBINED_TREES
   raw_nt = 0;
   usable_nt = 0;
   common_nt = 0;
#endif
l15:
  for (k = 1; (k <= me); k++)
    if (edge[k] == 1)
    {
/* this is the test that truncates the search space of the algorithm,
   requiring that no edges known to be essential to a properly qualified
   spanning tree ever be omitted from the trial edge list. these essential
   edges were seeded first in the initial edge list, so that the search
   pruning occurs as early as possible in the search space. all opamp
   edges and all port condition edges must be present in each properly
   qualified spanning tree.*/
      if((dmap[k] & ESSENTIAL_BRANCH_MASK) != 0)
      {
        l = k;           
        goto l60;
      }
      ;
      goto l40;
    }
  ;
l25:
  if (it <= 1) 
  {
    state = 2;
    return;
  }
  ;
  (it)--;

  for (k = 1; k <= me; k++)
    edge[k] = estk[it][k];

  for (k = 1, i = 0; k <= mn; k++)
  {
    v[k] = vstk[it][k];
    if (v[k] > i)
      i = v[k];
  }
  ;
  i++;
  goto l15;

l40:
  l = k;
  edge[l] = 0;
  m = 0;

  for (k = 1; k <= me; k++)
    if (edge[k] != 0)
      m++;

  if (m >= (mn - 1))
  {

    for (k = 1; k <= me; k++)
      estk[it][k] = edge[k];

    for (k = 1; k <= mn; k++)
      vstk[it][k] = v[k];

    (it)++;
  }
l60:
  edge[l] = -1;
  vd1 = di1[l];
  vd2 = di2[l];

  if (v[vd1] == 0)
    goto l95;
  if (v[vd2] == 0)
    goto l105;

  l = v[vd1];
  m = v[vd2];

  if (l == m)
    goto l25;
  if (l > m) 
  {
    k = m;
    m = l;
    l = k;
  }
  ;
  for (k = 1; k <= mn; k++) 
  {
    if (v[k] > m)
      v[k]--;
    else if (v[k] == m)
      v[k] = l;
  }
  ;
  i--;
  goto l110;
l95:
  if (v[vd2] == 0)
    goto l100;
  v[vd1] = v[vd2];
  goto l110;
l100:
  v[vd1] = i;
  v[vd2] = i;
  i++;
  goto l15;
l105:
  v[vd2] = v[vd1];
l110:
  for (k = 1; k <= mn; k++)
    if (v[k] != 1)
      goto l15;
  (nt)++;
#ifdef COMBINED_TREES
   raw_nt++;
#endif
  l = 1;

  if ((test & (RAW_TREES)) != 0) 
  {
    printf (" raw i-tree no. %5d edges:", nt);
    for (k = 2; k <= me; k++)
      if(edge[k] == -1) 
        printf ("%3d(%s) ", k,&branch[d0[k]][0]);
  }
  ;


/*
mq == 0 rejects all trees with any 'f' branch;
mq == 2 accepts all trees with only 'f1' branch;
mq == 4 accepts all trees with only 'f2' branch;
mq == 6 accepts all trees with both 'f1' and 'f2 branches */

  f0 = 0;
  for (k = 2,op = 0; k <= me; k++)	/* dummy edge 1 is always present, so skip it */
    if (edge[k] == -1) 
    {
      mtree[l] = k;
      l++;
      f0 |= dmap[k];
      op += (dmap[k] & 0x1);

    }
  ;

/* this test enforces the nullor constraint */
/* this might be redundant */

  if(opampcnt > op)
  {
    oprej++;
    if ((test & (RAW_TREES)) != 0)
      printf(" rejected: opamp sorting\n");
    goto l25;
  }
  ;

#ifdef COMBINED_TREES
   mtree[0] = (((f0 & TREEMASK) >> 1) + mq);
   if((mtree[0] == 4) || (mtree[0] == 7))
     goto l25;

   usable_nt++;
#endif

#ifndef COMBINED_TREES
  if ((f0 & TREEMASK) != mq) 
  {
    if ((test & (RAW_TREES)) != 0) 
    {
      printf("rejected: not a  %s-tree\n",(mq == TWOTREES)?"2":((mq == ONETREES)?"1":
        "3"));
    }
    ;
    goto l25;

  }
  ;
#endif
/* here is Rodanski's suggestion, that the spanning tree found in the i-graph is checked against 
   the v-graph. some facts to bear on this test: the v- and i-graphs have the same nodes, and the same 
   branches but the connectivity may differ, especially due to active elements and port condition edges;
   since the given tree is a spanning tree, it covers all nodes; the qualified common tree of the v-graph
   must also cover all nodes. if the i-tree edges do not form a spanning tree of the v-graph,
   then at least one node is isolated or not reachable (incidentally, this implies there is a
   circuit or loop) and there is no corresponding common v-tree. the nodes are marked as the tree 
   is traversed in the v-graph; any remaining unmarked
   nodes indicate lack of a common spanning tree in the v-graph. */
  {
/* this is the same traversal code as in the function treecheck */
/* note that v[] has been set to all 1's by the time execution reaches this code. */

    for (i = 1; i < l; i++) 
    {
      tag[i] = 0;
    }
    ;

    stack[0] = dv1[mtree[1]];
    v[stack[0]] = 0;
    depth = 0;
    for (done = 0; done == 0;) 
    {
      root_node = stack[depth];
      for (i = 1; i < l; i++) 
      {
        if (tag[i] == 1)
          continue;
        if (dv1[mtree[i]] == root_node) 
        {
          tag[i] = 1;
          princip_node = dv2[mtree[i]];
          break;
        }
        ;
        if (dv2[mtree[i]] == root_node) 
        {
          tag[i] = 1;
          princip_node = dv1[mtree[i]];
          break;
        }
        ;
      }
      ;
      if (i < l) 
      {
        depth++;
        stack[depth] = princip_node;
        v[princip_node] = 0;
      }
      else
      {
        if (depth > 0)
          depth--;
        else
          done = 1;
      }
      ;
    }
    ;
  }
  ;

  for (k = 1; k < mn; k++)
    if (v[k] != 0)
    {
      if ((test & (RAW_TREES)) != 0)
        printf(" rejected: no corresponding v-tree\n");
      goto l25;
    }

  if ((test & (RAW_TREES)) != 0)
    printf(" accepted\n");

#ifdef COMBINED_TREES
   common_nt++;
#endif

  return;

}

/* this sign weights each edge of the supplied tree based on its orientation
   relative to the established principal node in the traversal, and defines
   a traversal ordering that implies a further sign determining permutation.
   see Mayeda's book in the references */

void 
treecheck (int r, int *d1, int *d2, int *tree, int *perm)
{
  static   int stack[nmax], tag[nmax];
  static   int depth, i, j, done, root_node, princip_node;

  for (i = 1; i <= r; i++) 
  {
    tag[i] = 0;
  }
  ;

  perm[0] = 0;
  perm[1] = 0;
  stack[0] = 1;
  depth = 0;
  for (done = 0; done == 0;) 
  {
    root_node = stack[depth];
    for (i = 1; i <= r; i++) 
    {
      if (tag[i] == 1)
        continue;
      if (d1[tree[i]] == root_node) 
      {
        tag[i] = 1;
        princip_node = d2[tree[i]];
        perm[princip_node] = tree[i];
        break;
      }
      ;
      if (d2[tree[i]] == root_node) 
      {
        tag[i] = 1;
        princip_node = d1[tree[i]];
        perm[princip_node] = (-tree[i]);
        break;
      }
      ;
    }
    ;
    if (i <= r) 
    {
      depth++;
      stack[depth] = princip_node;
    }
    else
    {
      if (depth > 0)
        depth--;
      else
        done = 1;
    }
    ;
  }
  ;
  return;
}
/* for each topological formula, its mixed unit admittance tree products are collected and sorted here.
   the sign weighted tally of identical products is stored in termlist. identical branch labels are 
   combined as the power of the unique branch (name). each term in termlist is a list of powers of unique branch names.
   the terms are sorted for convienence in searching (binary search) for matching terms as they are generated.
   poly partitions the termlist based on the degree of s represented by the mixed unit factors of each term. the
   mixed unit representation is maintained until the numerator and denominator are associated as a ratio to form a
   two-port parameter function
 */
int
getexpr (int sign, int deg, int factors, int *factorlist, int edges, PARAMETERLIST)
{
  int i, j, k, l, m, n;
  int u,v;
  int (*poly)[maxdegree], (*termlist)[maxsym + 2];
  double (*termvalue);
  int tmpsize,tmpsize1,origsize;
  void *tmpptr;
  void *tmpptr1;

  poly = *ppoly;
  termlist = *ptermlist;

  for (i = 0; i < maxsym; i++)
    term[i] = 0;
/* collapse repeated identical branch names */
  for (j = 0; j < factors; j++) 
  {
    i = dbranchmap[factorlist[j]];
    term[i] = term[i] + 1;
  }
  ;
/*debug int denfn,numfn,inptno,evfn; */
  if ((test & (DEBUG_256)) != 0) 
  {
    printf ("%2d,%2d,%2d,%2d,%2d,%2d:", inptno, evfn, numfn, denfn, deg, sign);
    for (j = 0; j < factors; j++)
      printf (",%3d", factorlist[j]);
    printf ("\n");
  }
  ;
  if ((test & (DEBUG_512)) != 0)
    return (0);
/* merge new mixed unit admittance tree product into storage structure */
  if (degterms[deg] == 0) 
  {
/* the power of s is new for this topological function, so, insert this term as the first instance for this power (deg) */

/* save index to termlist for this first term of this deg */
    poly[0][deg] = *toterms;
    for (i = 0; i < maxsym; i++)
      termlist[*toterms][i] = term[i];
    termlist[*toterms][maxsym] = sign;
    termlist[*toterms][maxsym + 1] = 0;
    if (((*toterms)+1) >= (*maxtermlist)) 
    {
      {
        if ((test & (PRINT_STATS)) != 0)
          printf("resizing termlist ");
        tmpsize = (*maxtermlist);
        tmpsize1 = tmpsize;
        origsize = tmpsize;
        tmpptr = (*ptermlist);
        if(0 != resize((void *)&tmpptr, &tmpsize, sizeof (int) * (maxsym + 2)) )
        {
          return(1);
        }
        ;
        if ((test & (PRINT_STATS)) != 0)
		          printf("resizing termvalue ");
	tmpptr1 = (*ptermvalue);
        if(0 != resize((void *)&tmpptr1, &tmpsize1, sizeof (double)) )
        {
          return(1);
        }
        ;
        if (tmpsize != tmpsize1) 
        {
           return(1);
        };

        *ptermvalue = tmpptr1;
        termvalue = *ptermvalue;
        
        *ptermlist = tmpptr;
        *maxtermlist = tmpsize;
        termlist = *ptermlist;
      }
      for (i = origsize; i < (*maxtermlist); i++)
        {
        termvalue[i] = 0.0;
        for (j = 0; j <= (maxsym + 1); j++)
          termlist[i][j] = 0;
        };
    }
    (*toterms)++;
    degterms[deg] = 1;

  }
  else
  {
/* since the power of s is the same as earlier terms, do a binary search to find a matching term if possible */
    for (m = (degterms[deg]-1)/2, u = 0, v = (degterms[deg]-1),n = 0; (u <= v) && (n >= 0); m = (u+v)/2) 
    {


      k = poly[m][deg];
      for (i = 0; i < maxsym; i++)
      {
        if (termlist[k][i] < term[i])
        {
          v = m-1;
          n = m;
          break;
        }
        else
          if (termlist[k][i] > term[i])
        {
          u = m+1;
          n = m+1;
          break;
        }
        else
          n = -1;
      }
      ;
      if (n < 0) 
      {
/* found matching term, so tally weighted by term's sign */
        termlist[k][maxsym] += sign;

      }
      ;
    }
    ;
/* since no existing term matches, then insert the new term in this power partition. */
    if (n >= 0) 
    {
      k = degterms[deg];
      if(n < k)
        for(i=k; i>n; i--)
          poly[i][deg] = poly[i-1][deg];

      poly[n][deg] = *toterms;
      for (i = 0; i < maxsym; i++)
        termlist[*toterms][i] = term[i];
      termlist[*toterms][maxsym] = sign;
      termlist[*toterms][maxsym + 1] = 0;

      if (((*toterms)+1) >= (*maxtermlist)) 
      {
        {

          if ((test & (PRINT_STATS)) != 0)
            printf("resizing termlist ");
          tmpsize = (*maxtermlist);
        tmpsize1 = tmpsize;
          origsize = tmpsize;
          tmpptr = (*ptermlist);
          if(0 != resize((void *)&tmpptr, &tmpsize, sizeof (int) * (maxsym + 2)) )
          {
            return(1);
          }
          ;
        if ((test & (PRINT_STATS)) != 0)
		          printf("resizing termvalue ");
	tmpptr1 = (*ptermvalue);
        if(0 != resize((void *)&tmpptr1, &tmpsize1, sizeof (double)) )
        {
          return(1);
        }
        ;
        if (tmpsize != tmpsize1) 
        {
           return(1);
        };

        *ptermvalue = tmpptr1;
        termvalue = *ptermvalue;
        
          *ptermlist = tmpptr;
          *maxtermlist = tmpsize;
          termlist = *ptermlist;

        }
        for (i = origsize; i < (*maxtermlist); i++)
          for (j = 0; j <= (maxsym + 1); j++)
            termlist[i][j] = 0;

      }
      (*toterms)++;
      degterms[deg]++;
      if ((degterms[deg]+1) >= (*maxpoly)) 
      {
        {

          if ((test & (PRINT_STATS)) != 0)
            printf("resizing poly ");
          tmpsize = (*maxpoly);
          origsize = tmpsize;
          tmpptr = (*ppoly);

          if(0 != resize((void *)&tmpptr, &tmpsize, sizeof (int) * maxdegree) )
          {
            return(1);
          }
          ;
          *ppoly = tmpptr;
          *maxpoly = tmpsize;
          poly = *ppoly;
        }
        for (j = 0; j < maxdegree; j++)
          for (k = origsize; k < (*maxpoly); k++)
            poly[k][j] = 0;


      }
      ;


    }
    ;
  }
  ;

  return (0);
}

/* readjust structure sizes as more space is needed */
int resize(void ** ptr, int * count, int module)
{
  int tmpsize;
  void * tmpptr;
  if ((test & (PRINT_STATS)) != 0)
    printf ("orig size =%d, orig ptr =0x%08x\n",(*count) ,(*ptr) );
  tmpsize = 2 * (*count);
  tmpptr = (void *) realloc ((*ptr), tmpsize * module);

  if ((tmpptr != (void *) 0)) 
  {
    if ((test & (PRINT_STATS)) != 0)
      printf ("num of elements =%d, pointer =0x%08x\n", tmpsize, tmpptr);
    *ptr = tmpptr;
    *count = tmpsize;
    return(0);
  }
  else
  {
    tmpsize = (tmpsize + (*count)) / 2;
    while (tmpsize > (*count)) 
    {
      tmpptr = (void *) realloc ((*ptr), tmpsize * module);

      if ((tmpptr != (void *) 0)) 
      {
        if ((test & (PRINT_STATS)) != 0)
          printf ("num of elements =%d, pointer =0x%08x\n", tmpsize, tmpptr);
        *ptr = tmpptr;
        *count = tmpsize;
        return(0);
      }
      else
      {
        tmpsize = (tmpsize + (*count)) / 2;
        if (tmpsize == (*count))
        {
          if ((test & (PRINT_STATS)) != 0)
            printf("resize failed for insufficient memory\n");
          return (1);
        }
        ;
      }
      ;
    }
  }
  ;
  printf("resize failed for insufficient memory\n");
  return(1);

}


/* combines the topological functions for numerator and denominator, and applies necessary
   modifications to the terms of each; finally prints numerator and denominator */
int
print_treebuf (int sign, int num_fun, int denom_fun)
{
if ((test & (PRINT_TRACE)) != 0)
printf("print_tree_buf entered\n");


  if ((test & (PRINT_STATS | RAW_TREES | TREE_PERM | MIXED_UNITS | ADMITT_UNITS | DEBUG_128)) != 0) 
  {
    printf("Topological formula: %c(%s, %1d-trees) / (%s, %1d-trees)\n",
      (sign > 0)?' ':'-',
      topo_names[num_fun],(num_fun ==0)?1:((num_fun == 5)?3:
      2),
      topo_names[denom_fun],(denom_fun ==0)?1:((denom_fun == 5)?3:
      2));
  }
  ;
 if ((test & (PRINT_TRACE)) != 0)
printf("copy numerator called\n");
 if(0 != copy(sign, numerator, num_fun) )
    return(1);
if ((test & (PRINT_TRACE)) != 0)
printf("copy denominator called\n");
  if(0 != copy( 1, denominator, denom_fun) )
    return(1);




  if ((test & (DEBUG_512)) != 0) 
  {
    release (NUM_VARS);
    release (DENOM_VARS);
    printf ("\n End:\n\n");
    return(0);
  }
  ;

if ((test & (PRINT_TRACE)) != 0)
printf("combineinit called\n");
  combineinit();
if ((test & (PRINT_TRACE)) != 0)
printf("fixup called\n");
  if(0 != fixup () )
  {
    printf("insufficient memory to complete processing\n");
    printf ("\n Numerator:\n");
    printf ("\n Denominator:\n");
    printf ("\n End:\n\n");
    release (NUM_VARS);
    release (DENOM_VARS);
    return(1);
  }
  ;

  printf ("\n Numerator:\n");

  numcntxt = 0;
  printexp (NUM_VARS);



  printf ("\n Denominator:\n");

  numcntxt = 1;
  printexp (DENOM_VARS);

  release (NUM_VARS);
  release (DENOM_VARS);


  printf ("\n End:\n\n");

  return(0);

}

int copy(int sign, int dest, int source)
{
  int i, j, k;
  int (*polys)[maxdegree], (*termlists)[maxsym + 2];
  double (*termvalues);
  int (*polyd)[maxdegree], (*termlistd)[maxsym + 2];
  double (*termvalued);

  vars[dest].termlist = (void *) malloc ((vars[source].maxterms) * sizeof (int) * (maxsym + 2));

  if (0 == vars[dest].termlist)
  {
    printf("can't allocate space for termlist\n");
    return(1);
  }
  ;

  vars[dest].termvalue = (void *) malloc ((vars[source].maxterms) * sizeof (double));

  if (0 == vars[dest].termvalue)
  {
    printf("can't allocate space for termvalue\n");
    return(1);
  }
  ;

  vars[dest].poly = (void *) malloc ((vars[source].maxpoly) * sizeof (int) * maxdegree);

  if (0 == vars[dest].poly)
  {
    free(vars[dest].termlist);
    printf("can't allocate space for poly\n");
    return(1);
  }
  ;


  polyd = vars[dest].poly;
  termlistd = vars[dest].termlist;
  termvalued = vars[dest].termvalue;
  polys = vars[source].poly;
  termlists = vars[source].termlist;
  termvalues = vars[source].termvalue;

  for (j = 0; j < maxdegree; j++)
    for (k = 0; k < (vars[source].maxpoly); k++)
      polyd[k][j] = polys[k][j];

  vars[dest].maxpoly = vars[source].maxpoly;

  for (i = 0; i < (vars[source].maxterms); i++)
    {
      termvalued[i] = termvalues[i];
      for (j = 0; j <= (maxsym + 1); j++)
        termlistd[i][j] = termlists[i][j];
    };

  if (sign < 0)
    for (i = 0; i < (vars[source].maxterms); i++)
      termlistd[i][maxsym] = (-termlistd[i][maxsym]);

  vars[dest].maxterms = vars[source].maxterms;

  for (j = 0; j < maxdegree; j++)
    vars[dest].degterms[j] = vars[source].degterms[j];

  vars[dest].toterms = vars[source].toterms;


  return(0);

}

/* prepare to divide out the impedance terms in numerator and denominator
   to straighten out the mixed units. */

void
combineinit (void)
{
  int i;


  for (i = 0; i < maxsym; i++)
    cancelfactor[i] = 0x7fffffff;

  cancelcoeff = 0;
  cancelsign = -1;
  cancelexp = 0x7fffffff;


}

/* branchtag accumulates the powers of the impedance factors that
   must divide both the numerator and denominator in order to
   correct the mixed unit representation. cancelfactor has branch
   factors that divide all terms. The appendix in Percival's paper
   describes the use and correction of mixed unit tree products.
*/
int 
fixup (void)
{
  int j, k;

if ((test & (PRINT_TRACE)) != 0)
printf("fixup entered, eval_branchtag for numerator called\n");
  eval_branchtag (NUM_VARS);
if ((test & (PRINT_TRACE)) != 0)
printf("eval_branchtag for denominator called\n");

  eval_branchtag (DENOM_VARS);

  for (j = 1; j <= (e + a); j++) 
  {
    if (branchtag[j] == 0)
      continue;
    else
    {
      k = branchmap[j] /*getsym (j, (e + a))*/;
      if (j > k) 
      {
        branchtag[j] = 0;
        branchtag[k]++;
      }
      ;
    }
    ;
  }
  ;

  if ((test & (MIXED_UNITS)) != 0) 
  {
    printf ("these ohm-unit factors modify all terms of both num and denom:\n");
    for (k = 1; k <= (e + a); k++)
      if ((branchtag[k]) > 0) 
      {
        if ((cancelfactor[k]) > 1) /* this looks out of place: cancelfactor isn't evaluated until makecancellationlist */
          printf (" %s^%-2d", &branch[k][0], (branchtag[k]));
        else
          printf (" %s   ", &branch[k][0]);
      }
    ;

    printf ("\n");
    for (j = 1; j <= (e + a); j++)
      branchtag[j] = 0;
    printf("note that s*L, s*M, and R represent s/L, s/M, and 1/R in the following function:\n");
  }
  ;
if ((test & (PRINT_TRACE)) != 0)
printf("correctunits for numerator called\n");
  if(0 != correctunits (NUM_VARS) )
    return(1);
if ((test & (PRINT_TRACE)) != 0)
printf("correctunits for denominator called\n");
  if(0 != correctunits (DENOM_VARS) )
    return(1);
if ((test & (PRINT_TRACE)) != 0)
printf("makecancellationlist for numerator called\n");
  makecancellationlist (NUM_VARS);
if ((test & (PRINT_TRACE)) != 0)
printf("makecancellationlist for denominator called\n");
  makecancellationlist (DENOM_VARS);

if ((test & (PRINT_TRACE)) != 0)
printf("evaluateliterals for numerator called\n");
  evaluateliterals (NUM_VARS);
if ((test & (PRINT_TRACE)) != 0)
printf("evaluateliterals for denominator called\n");
  evaluateliterals (DENOM_VARS);

  return(0);

}


void 
printexp (PARAMETERLIST)
{
  int i, j, k, l, m, n;

  int (*poly)[maxdegree], (*termlist)[maxsym + 2];
  double (*termvalue);

  poly = *ppoly;
  termlist = *ptermlist;
  termvalue = *ptermvalue;


/*return;*/

  for (i = 0, n = 0; i < maxdegree; i++) 
  {
    if (degterms[i] > 0) 
    {
      for (j = 0, k = 0; (j < degterms[i]) && (k == 0); j++) 
      {
        l = poly[j][i];
        if ((termvalue[l] != 0))
          k = 1;

      }
      ;

      if (k == 1) 
      {
        n++;
        printf ("\n + s^%-2d (\n", i - cancelexp);
        for (j = 0; j < degterms[i]; j++) 
        {
          l = poly[j][i];
          if ((termvalue[l] == 0))
            continue;

            printf ("         ");

           printf ("%g", termvalue[l]);

          for (k = 0, m = 0; k < maxsym; k++) 
          {
            if ((termlist[l][k]) > 0) 
            { 
              m = 1;
              if ((termlist[l][k]) > 1)
                printf (" %s^%-2d", &branch[k][0], (termlist[l][k]));
              else
                printf (" %s   ", &branch[k][0]);
             };
          }
            ;
           printf("\n");
         }
          ;
          printf ("        )");
        }
        ;
      }
      ;
    }
    ;
    if (n == 0)
    printf ("\n        0");
  printf ("\n");
}





void 
eval_branchtag (PARAMETERLIST)
{
/* branchtag collects all the impedance-unit factors to be divided out later */

  int i, j, k, l, m, n;
  int (*poly)[maxdegree], (*termlist)[maxsym + 2];


  poly = *ppoly;
  termlist = *ptermlist;


  if ((test & (DEBUG_512)) != 0)
    return;


  for (i = 0, n = 0; i < maxdegree; i++) 
  {
    if (degterms[i] > 0) 
    {
      for (j = 0; (j < degterms[i]); j++) 
      {
        l = poly[j][i];
        if ((termlist[l][maxsym] != 0)) 
        {
          for (k = 0; k < impedancecount; k++) 
          {
            m = impedancelist[k];
            if (termlist[l][m] > branchtag[m])
              branchtag[m] = termlist[l][m];
          }
          ;
        }
        ;

      }
      ;
    }
    ;
  }
  ;



  return;
}




/* the original mixed unit terms are re-evaluated using the branchtag information. the power of s
   of the resulting terms is recomputed and the terms are moved to the proper power partition as required. */
int 
correctunits (PARAMETERLIST)
{
  int i, ii, j, k, l, m, n, s;
  int kk;
  int u,v;
  int (*poly)[maxdegree], (*termlist)[maxsym + 2];
  int tmpsize, origsize;
  void *tmpptr;


  poly = *ppoly;
  termlist = *ptermlist;



  for (i = 0; i < maxdegree; i++) 
  {
    if (degterms[i] > 0) 
    {

/* find at least one term for this degree that hasn't been corrected yet */
      for (j = 0, k = 0; (j < degterms[i]) && (k == 0); j++) 
      {
        l = poly[j][i];
        if (		/*(termlist[l][maxsym] != 0)&& */
          (termlist[l][maxsym + 1] == 0))
          k = 1;

      }
      ;

      if (k == 1) 
      {
        for (j = 0; j < degterms[i]; j++) 
        {
          l = poly[j][i];
          if (	/*(termlist[l][maxsym] == 0) || */
            (termlist[l][maxsym + 1] == 1))
            continue; /* this term already examined or changed */

/* tag this term as being examined/changed now */
          termlist[l][maxsym + 1] = 1;


/* the division by the impedance unit factors is done in this loop:
   this difference of exponents of factors is never negative */
          for (k = 0; k < maxsym; k++) 
          {
            if ((branchtag[k] != 0)) 
            {
              termlist[l][k] = branchtag[k] - termlist[l][k];
            }
            ;
          }
          ;
/* evaluate the new power of s for the corrected term */

          for (k = 0, s = 0; k < maxsym; k++) 
          {
            if (powermap[k] != 0)
              s += termlist[l][k];
          }
          ;
/* when the new power of s is different than the original (uncorrected) term, it 
   belongs in a different partition of terms */

          if (s != i) 
          {
            if (degterms[s] == 0) 
            {
              poly[0][s] = l;
              degterms[s] = 1;

            }
            else
            {
              for (m = 0, n = 0; (m < degterms[s]) && (n == 0); m++) 
              {
                k = poly[m][s];
                if (termlist[k][maxsym + 1] == 0)
                  continue;
                for (ii = 0; ii < maxsym; ii++)
                  if (termlist[k][ii] != termlist[l][ii])
                    break;
                if (ii == maxsym) 
                {
                  n = 1;
                  termlist[k][maxsym] += termlist[l][maxsym];

                }
                ;
              }
              ;
              if (n == 0) 
              {
                k = degterms[s];
                if ((degterms[s]+1) >= (*maxpoly)) 
                {
                  {





                    if ((test & (PRINT_STATS)) != 0)
                      printf("resizing poly ");
                    tmpsize = (*maxpoly);
                    origsize = tmpsize;
                    tmpptr = (*ppoly);

                    if(0 != resize((void *)&tmpptr, &tmpsize, sizeof (int) * maxdegree) )
                    {
                      return(1);
                    }
                    ;
                    *ppoly = tmpptr;
                    *maxpoly = tmpsize;
                    poly = *ppoly;
                  }
                  for (ii = 0; ii < maxdegree; ii++)
                    for (n = origsize; n < (*maxpoly); n++)
                      poly[n][ii] = 0;

                }
                ;
                poly[k][s] = l;
                degterms[s]++;


              }
              ;
            }
            ;
            if ((j + 1) < degterms[i])
              for (ii = (j + 1); ii < degterms[i]; ii++)
                poly[ii - 1][i] = poly[ii][i];
            degterms[i]--;
            j--;
          }
          ;
        }
        ;
      }
      ;
    }
    ;
  }
  ;


/* combine any duplicate terms that have resulted from above correction */

  for (i = 0; i < maxdegree; i++) 
  {
    if (degterms[i] > 0) 
    {
      for (j = (degterms[i] - 1); (j >= 0); j--) 
      {
        l = poly[j][i];
        if (		/*(termlist[l][maxsym] != 0)&& */
          (termlist[l][maxsym + 1] == 1)) 
        {

          for(v=0,u=(j-1),k=((j-1)/2);(u>=v);k=((u+v)/2))
          {
            m = poly[k][i];

            for (ii = 0; ii < maxsym; ii++)
            {
              if (termlist[l][ii] < termlist[m][ii])
              {
                v = k + 1;
                break;
              }
              else
                if (termlist[l][ii] > termlist[m][ii])
              {
                u = k - 1;
                break;
              }
              ;

            }
            ;

            if (ii == maxsym) 
            {
/* found two terms with the same factors, so combine them */

              termlist[m][maxsym] += termlist[l][maxsym];
              if ((j + 1) < degterms[i])
                for (ii = (j + 1); ii < degterms[i]; ii++)
                  poly[ii - 1][i] = poly[ii][i];
              degterms[i]--;

              break;
            }
            ;




          }
          ;


        }
        ;

      }
      ;
    }
    ;
  }
  ;

  return(0);


}

int 
evaluateliterals (PARAMETERLIST)
{
  int i, ii, j, k, l, m, n, s;
  int kk,newdegterms;
  int u,v;
  int (*poly)[maxdegree], (*termlist)[maxsym + 2];
  double (*termvalue);

  int tmpsize, origsize;
  void *tmpptr;


  poly = *ppoly;
  termlist = *ptermlist;
  termvalue = *ptermvalue;
{

  if (cancelcoeff == 0)
    cancelcoeff = 1;

  if ((test & (ADMITT_UNITS)) != 0) 
  {
    printf ("common factors to cancel in all terms of both num and denom:\n");
    printf ("%s%d ", cancelsign > 0 ? "+" : "-", cancelcoeff);
    for (k = 0; k < maxsym; k++)
      if ((cancelfactor[k]) > 0) 
      {
        if ((cancelfactor[k]) > 1)
          printf (" %s^%-2d", &branch[k][0], (cancelfactor[k]));
        else
          printf (" %s   ", &branch[k][0]);
      }
    ;

    printf ("\n");
  }
  ;



  for (i = 0, n = 0; i < maxdegree; i++) 
  {
    if (degterms[i] > 0) 
    {
      for (j = 0, k = 0; (j < degterms[i]) && (k == 0); j++) 
      {
        l = poly[j][i];
        if ((termlist[l][maxsym] != 0))
          k = 1;

      }
      ;

      if (k == 1) 
      {
        n++;
        for (j = 0; j < degterms[i]; j++) 
        {
          l = poly[j][i];

          
          {/* new */
               termvalue[l] =  cancelsign * termlist[l][maxsym] / cancelcoeff;
          for (k = 0, m = 0; k < maxsym; k++) 
          {
            if ((termlist[l][k] - cancelfactor[k]) > 0) 
            { for (m = 0; m < (termlist[l][k] - cancelfactor[k]); m++)
              termvalue[l] = termvalue[l] * ((value[k] != 0)?value[k]:1.0);
            }
            ;
            if ((termlist[l][k] == cancelfactor[k]) || (value[k] != 0)) 
             {
              termlist[l][k] = 0;
             }else{
              termlist[l][k] = termlist[l][k] - cancelfactor[k];
             };
          }
          ;
          };

        }
        ;
      }
      ;
    }
    ;
  }
  ;





}



/* combine any duplicate terms that have resulted from above correction */

  for (i = 0; i < maxdegree; i++) 
  {
    if (degterms[i] > 1) 
    {
      newdegterms = 1;
      for (j = 1; j <degterms[i]; j++) 
      {
        l = poly[j][i];
        {

       for(v=0,u=(newdegterms-1),k=((newdegterms-1)/2),n=0;(u>=v) && (n >= 0);k=((u+v)/2))
          {
            m = poly[k][i];
           /* n = -1;*/

            for (ii = 0; ii < maxsym; ii++)
            {
                   if (value[ii] != 0) 
		   {
                   n = -1;
                   continue;
                   };
              if (termlist[l][ii] < termlist[m][ii])
              {
                v = k + 1;
		n = k + 1;
                break;
              }
              else
                if (termlist[l][ii] > termlist[m][ii])
              {
                u = k - 1;
		n = k;
                break;
              }
              else
              n = -1;

	    };

          }
            ;
	    if (n < 0)
	     termvalue[m] += termvalue[l];

	    if (n >= 0)
            {
                    k = newdegterms;
                    ii = poly[j][i];
                  if (n < k)
		    for (u=k;u>n;u--)
			    poly[u][i] = poly[u-1][i];

                    poly[n][i] = ii;
		    newdegterms++;
            };




        }
        ;

      }
      ;
      degterms[i] = newdegterms;
    }
    ;
  }
  ;

  return(0);


}

/* the common branch factors and gcd of term coefficients is determined in order
   to cancel them out during  printing 
   this is called twice, once for the numerator and the second time with the denominator 
   the cancellation data is combined for both */

void 
makecancellationlist (PARAMETERLIST)
{
  int i, j, k, l, m, n;

  int (*poly)[maxdegree], (*termlist)[maxsym + 2];


  poly = *ppoly;
  termlist = *ptermlist;

  for (i = 0; i < maxdegree; i++) 
  {
    if (degterms[i] > 0) 
    {
      for (j = 0, k = 0; (j < degterms[i]) && (k == 0); j++) 
      {
        l = poly[j][i];
        if ((termlist[l][maxsym] != 0))
          k = 1;

      }
      ;

      if (k == 1) 
      {
        if (cancelexp > i)
          cancelexp = i;
        for (j = 0; j < degterms[i]; j++) 
        {
          l = poly[j][i];
          if ((termlist[l][maxsym] == 0))
            continue;

/* the common branches are found here */
          for (k = 0; k < maxsym; k++) 
          {
            if ((termlist[l][k] < cancelfactor[k]))
              cancelfactor[k] = termlist[l][k]; /* the least factor is all that may be cxancelled */
          }
          ;

/* common sign */
          if ((termlist[l][maxsym] > 0))
            cancelsign = 1;

/* common coefficient */
          if (((n = termlist[l][maxsym]) < 0))
            n = -n;
          if ((cancelcoeff > 0) && (cancelcoeff != n)) 
          {
            if (cancelcoeff > n) 
            {
              m = n;
              n = cancelcoeff;
            }
            else
              m = cancelcoeff;

            for (; n > m;) 
            {
              for (; n > m;)
                n -= m;
              l = m;
              m = n;
              n = l;
            }
            ;
            cancelcoeff = n;
          }
          else
            cancelcoeff = n;
        }
        ;
      }
      ;
    }
    ;
  }
  ;
} 
